-------------------------------------------------------------------------------------
-- Bug 45840 - EET - umožnit zakázat pokus o vícenásobné současné odeslání EET tržby 
-------------------------------------------------------------------------------------

CREATE SEQUENCE seq_wlocks
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


ALTER TABLE public.seq_wlocks OWNER TO %firmaRole%;

CREATE TABLE wlocks (
  idlock integer DEFAULT nextval('seq_wlocks'::regclass) NOT NULL,
  idzaznamu integer,
  beanKey varchar (100),
  typ varchar (50) not null,
  doba integer not null Default 0,
  platiDo timestamp not null
);

ALTER TABLE public.wlocks OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION wlocksfillplatiDo()
  RETURNS trigger AS $$
begin
  if(NEW.platiDo is null) then
    NEW.platiDo = now() + (interval '1 second' * NEW.doba);
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.wlocksfillplatiDo() OWNER TO %firmaRole%;

CREATE TRIGGER wlocksfillplatiDo BEFORE INSERT ON wlocks
  FOR EACH ROW EXECUTE PROCEDURE wlocksfillplatiDo();

ALTER TABLE wlocks
  ADD CONSTRAINT wlocks_idzaznamu_beanKey_typ_key UNIQUE(idzaznamu, beanKey, typ);

CREATE OR REPLACE FUNCTION iszaznamlocked(_idZaznamu integer, _beanKey varchar(100), _typ varchar(50))
  RETURNS boolean AS
$BODY$ 
declare 
  _pocetZaznamu integer;
begin
  -- nejdříve smažeme staré
  delete from wlocks
    where idZaznamu = _idZaznamu
      and beanKey = _beanKey
      and typ = _typ
      and platiDo < now();
      
  -- pak zjistíme jestli nějaké zbyly 
  select count(*) into _pocetZaznamu from wlocks 
    where idZaznamu = _idZaznamu
      and beanKey = _beanKey
      and typ = _typ;
 
  return _pocetZaznamu > 0; 
end; 
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION iszaznamlocked(_idZaznamu integer, _beanKey varchar(100), _typ varchar(50)) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 1043, 't', %appVersion%, clock_timestamp());