-------------------------------------------------------------------------------------
-- Bug 13984 – [WinStrom] Pomalá práce u podpisu či loga o velikosti stovky kilobytů.
-------------------------------------------------------------------------------------

-- vytvoreni tabulky pro data priloh

CREATE SEQUENCE seq_wprilohadata
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_wprilohadata OWNER TO %firmaRole%;

CREATE TABLE wprilohadata (
    idPrilohaData integer DEFAULT nextval('seq_wprilohadata'::regclass) NOT NULL,
    byteData bytea,
    blobData oid,
    CONSTRAINT wprilohadata_pkey PRIMARY KEY (idPrilohaData)
);
ALTER TABLE wprilohadata OWNER TO %firmaRole%;

-- vytvoreni tabulky pro popisek priloh

CREATE SEQUENCE seq_wpriloha
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_wpriloha OWNER TO %firmaRole%;

CREATE TABLE wpriloha (
  idpriloha integer DEFAULT nextval('seq_wpriloha'::regclass) NOT NULL,
  lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,
  nazsoub character varying(255) NOT NULL,
  poznam text,
  contenttype character varying(50) NOT NULL,
  preview bytea,
  typk character varying(50) NOT NULL,
  iddoklfak integer,
  iddoklint integer,
  iddoklobch integer,
  iddoklsklad integer,
  iduziv integer,
  iddata integer NOT NULL,
  CONSTRAINT wpriloha_pkey PRIMARY KEY (idpriloha),
  CONSTRAINT wpriloha_iduziv FOREIGN KEY (iduziv)
      REFERENCES wuzivatel (iduzivatel) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT wpriloha_iddata FOREIGN KEY (iddata)
        REFERENCES wprilohadata (idPrilohaData) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT wpriloha_iddoklfak FOREIGN KEY (iddoklfak) 
        REFERENCES ddoklfak(iddoklfak) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT wpriloha_iddoklint FOREIGN KEY (iddoklint)
        REFERENCES ddoklint(iddoklint) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT wpriloha_iddoklobch FOREIGN KEY (iddoklobch)
        REFERENCES ddoklobch(iddoklobch) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT wpriloha_iddoklsklad FOREIGN KEY (iddoklsklad)
        REFERENCES ddoklsklad(iddoklsklad) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE wpriloha OWNER TO %firmaRole%;

-- prekopirovani loga, podpisu a razitka do novych struktur + update odkazu v nastaveni

CREATE OR REPLACE FUNCTION copyLogoToNewSchema() RETURNS void AS
$BODY$
declare
	logo record;
	podraz record;
begin
  -- prekopirovani log
  for logo in select * from nlogo l where 
	exists (select 1 from nnastav where idloga = l.idlogo) order by idlogo asc loop
    insert into wprilohadata (idprilohadata, bytedata) values
	(nextval('seq_wprilohadata'), logo.image);

    insert into wpriloha (idpriloha, nazsoub, contenttype, typk, iddata) values
	(nextval('seq_wpriloha'), 'logo', 'image/jpeg', 'typPrilohy.logo', currval('seq_wprilohadata'));

    update nnastav set idloga = currval('seq_wpriloha') where idloga = logo.idlogo;
  end loop;

  -- prekopirovani podpisu a razitek
  for logo in select * from nlogo p where
	exists (select 1 from nnastav where idpodraz = p.idlogo) order by idlogo asc loop
    insert into wprilohadata (idprilohadata, bytedata) values
	(nextval('seq_wprilohadata'), logo.image);
	
    insert into wpriloha (idpriloha, nazsoub, contenttype, typk, iddata) values
	(nextval('seq_wpriloha'), 'podpis-razitko', 'image/jpeg', 'typPrilohy.podRaz', currval('seq_wprilohadata'));

    update nnastav set idpodraz = currval('seq_wpriloha') where idpodraz = logo.idlogo;
  end loop;	
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION copyLogoToNewSchema() OWNER TO %firmaRole%;

-- jeste musime "vypnout" constraint z nastaveni logo
alter table nnastav drop constraint fk45946497d96cb6cc;
alter table nnastav drop constraint nnastav_podraz_fk;

-- funkci zavolame zde dokud jsou vyple constraint na logo a nezaple na prilohu
select copyLogoToNewSchema();

alter table nnastav add constraint nnastav_logo_fk FOREIGN KEY (idloga)
	REFERENCES wpriloha (idpriloha) MATCH SIMPLE
	ON UPDATE NO ACTION ON DELETE NO ACTION;
alter table nnastav add constraint nnastav_podraz_fk FOREIGN KEY (idpodraz)
	REFERENCES wpriloha (idpriloha) MATCH SIMPLE
	ON UPDATE NO ACTION ON DELETE NO ACTION;

-- prekopirovani priloh k dokladum do novych struktur
CREATE OR REPLACE FUNCTION copyPrilohyToNewSchema() RETURNS void AS
$BODY$
declare
	prilohaRec record;
begin
  for prilohaRec in select * from dprilohy loop
    insert into wprilohadata (idprilohadata, blobdata) values
	(nextval('seq_wprilohadata'), prilohaRec.priloha);
	
    insert into wpriloha (idpriloha, nazsoub, contenttype, typk, iddoklfak, iddata) values
	(nextval('seq_wpriloha'), '', 'application/octet-stream', 'typPrilohy.doklad', prilohaRec.iddoklad, currval('seq_wprilohadata'));
  end loop;
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION copyPrilohyToNewSchema() OWNER TO %firmaRole%;

select copyPrilohyToNewSchema();

-- smazani nyni nepouzivanych tabulek a funkci
drop table nlogo;
drop table dprilohy;
drop function copyLogoToNewSchema();
drop function copyPrilohyToNewSchema();

-- triger na mazani priloh k mazanym fakturam je jiz hotov jenom pozmenime
-- funkci takze bude mazat nad jinou tabulkou
-- Trigger mazající přílohy při smazání dokladu
CREATE OR REPLACE FUNCTION ddoklfakdeleteprilohy()
  RETURNS trigger AS $$
begin
  delete from wpriloha where iddoklfak = OLD.idDoklFak;
  
  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ddoklfakdeleteprilohy() OWNER TO %firmaRole%;

insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 106, 't');
