-------------------------------------------------------------------------------------
-- Bug 13984 – [WinStrom] Pomalá práce u podpisu či loga o velikosti stovky kilobytů.
-------------------------------------------------------------------------------------

ALTER TABLE wpriloha ADD COLUMN idnastav integer;

ALTER TABLE wpriloha ADD constraint wpriloha_nastaveni_fk FOREIGN KEY (idnastav)
        REFERENCES nnastav (idnastav) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE OR REPLACE FUNCTION connectLogaWithNastaveni() RETURNS void AS
$BODY$
declare 
        priloha record;
        nastav record;
        _idnastav integer;
begin
  for priloha in select * from wpriloha p where 
        exists (select idnastav from nnastav where idloga = p.idpriloha) loop
    -- prvni nastaveni co ukazuje na prilohu nastavime rovnou
    select (select idnastav from nnastav where idloga = priloha.idpriloha limit 1) into _idnastav;
        
    update wpriloha set idnastav = _idnastav where idpriloha = priloha.idpriloha;

    update nnastav set idloga = null where idnastav = _idnastav;
    
    -- pro ostatni nastaveni musime provest kopii prilohy, ta bude ukazovat na tohle nastaveni
    for nastav in select * from nnastav n where idloga = priloha.idpriloha loop
        insert into wpriloha (idpriloha, nazsoub, poznam, contenttype, preview, typk, iddoklfak, iddoklint, 
        iddoklobch, iddoklsklad, iduziv, iddata, idnastav) values
	        (nextval('seq_wpriloha'), priloha.nazsoub, priloha.poznam, priloha.contenttype, priloha.preview,
	        priloha.typk, priloha.iddoklfak, priloha.iddoklint, priloha.iddoklobch, priloha.iddoklsklad,
	        priloha.iduziv, priloha.iddata, nastav.idnastav);
	-- nastavime nastaveni tak, ze uz neukazuje na logo
        update nnastav set idloga = null where idnastav = nastav.idnastav;
    end loop;
    
  end loop;
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION connectLogaWithNastaveni() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION connectPodrazWithNastaveni()  RETURNS void AS
$BODY$
declare 
        priloha record;
        nastav record;
        _idnastav integer;
begin
  for priloha in select * from wpriloha p where 
        exists (select idnastav from nnastav where idpodraz = p.idpriloha) loop
    -- prvni nastaveni co ukazuje na prilohu nastavime rovnou
    select (select idnastav from nnastav where idpodraz = priloha.idpriloha limit 1) into _idnastav;
        
    update wpriloha set idnastav = _idnastav where idpriloha = priloha.idpriloha;

    update nnastav set idpodraz = null where idnastav = _idnastav;
    
    -- pro ostatni nastaveni musime provest kopii prilohy, ta bude ukazovat na tohle nastaveni
    for nastav in select * from nnastav n where idpodraz = priloha.idpriloha loop
        insert into wpriloha (idpriloha, nazsoub, poznam, contenttype, preview, typk, iddoklfak, iddoklint, 
        iddoklobch, iddoklsklad, iduziv, iddata, idnastav) values
	        (nextval('seq_wpriloha'), priloha.nazsoub, priloha.poznam, priloha.contenttype, priloha.preview,
	        priloha.typk, priloha.iddoklfak, priloha.iddoklint, priloha.iddoklobch, priloha.iddoklsklad,
	        priloha.iduziv, priloha.iddata, nastav.idnastav);
	-- nastavime nastaveni tak, ze uz neukazuje na podraz
        update nnastav set idpodraz = null where idnastav = nastav.idnastav;
    end loop;
    
  end loop;
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION connectPodrazWithNastaveni() OWNER TO %firmaRole%;

select connectLogaWithNastaveni();
select connectPodrazWithNastaveni();

drop function connectLogaWithNastaveni();
drop function connectPodrazWithNastaveni();

ALTER TABLE nnastav DROP COLUMN idloga;
ALTER TABLE nnastav DROP COLUMN idpodraz;

insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 110, 't');
