-- Bug 11824 – [WinStrom] Účtují se i doklady, které nemají příznak zaúčtováno nastaven na true

-- oductovani chybne zauctovanych faktur
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
       -P.sumZkl, -P.sumZklMen) from dpolfak as P left outer join ddoklfak as D on D.idDoklFak = P.idDoklFak where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
       -P.sumDph, -P.sumDphMen) from dpolfak as P left outer join ddoklfak as D on D.idDoklFak = P.idDoklFak where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
       
-- oductovani chybne zauctovanych internich dokladu
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
       -P.sumZkl, -P.sumZklMen) from dpolint as P left outer join ddoklint as D on D.idDoklInt = P.idDoklInt where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
       -P.sumDph, -P.sumDphMen) from dpolint as P left outer join ddoklint as D on D.idDoklInt = P.idDoklInt where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
                                
                                
-- oductovani chybne zauctovanych skladovych dokladu                     
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
       -P.sumZkl, -P.sumZklMen) from dpolsklad as P left outer join ddoklsklad as D on D.idDoklSklad = P.idDoklSklad where
       D.zuctovano = false and P.ucetni = true and P.storno = false;
       
select createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
       -P.sumDph, -P.sumDphMen) from dpolsklad as P left outer join ddoklsklad as D on D.idDoklSklad = P.idDoklSklad where
       D.zuctovano = false and P.ucetni = true and P.storno = false;                                
                                
                                
-- funkce vrati koncovou cast nazvu tabulky dokladu/polozky pro dany modul (Fak/Int/Sklad...)
CREATE OR REPLACE FUNCTION getDoklNamePostfixByModul(modul varchar(20))
  RETURNS varchar(20) AS $$
declare
  _postfix varchar(20);
begin
  if (modul in( 'FAV','FAP','PHL','ZAV')) then
    _postfix := 'Fak';
  else
    if (modul in( 'SKL')) then
      _postfix := 'Sklad';
    else
      if (modul in( 'BAN','POK','INT')) then
        _postfix := 'Int';
      else
        if (modul in ('PPP', 'NAV', 'OBP', 'PPV', 'NAP', 'OBV')) then
          _postfix := 'Obchod';
        else
          raise exception 'getDoklNamePostfixByModul(): Neznámý modul: %', modul;
        end if;
      end if;
    end if;
  end if;
  
  return _postfix;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.getDoklNamePostfixByModul(varchar) OWNER TO %firmaRole%;


-- oprava triggeru ktery uctuje pri vytvoreni polozky
CREATE OR REPLACE FUNCTION dpoldoklinsertucto()
  RETURNS trigger AS $$
declare
  _postfix varchar(20);
  _idDokl bigint;
  _zuctovano boolean;
begin
  select getDoklNamePostfixByModul(NEW.modul) into _postfix;
  if (_postfix = 'Fak') then
    _idDokl := NEW.idDoklFak;
  else
    if (_postfix = 'Int') then
      _idDokl := NEW.idDoklInt;
    else
      if (_postfix = 'Sklad') then
        _idDokl := NEW.idDoklSklad;
      end if;
    end if;
  end if;
  
  execute 'select zuctovano from dDokl' || _postfix || ' where idDokl' || _postfix || ' = ' || _idDokl into _zuctovano;

  if (_zuctovano) then
    execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,  
        NEW.idMeny, NEW.idStred, NEW.sumZkl, NEW.sumZklMen);  
    
    execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,  
        NEW.idMeny, NEW.idStred, NEW.sumDph, NEW.sumDphMen);
  end if;  
  
  return NEW;  
end;  
$$ LANGUAGE plpgsql;


-- oprava triggeru ktery uctuje pri update polozky
CREATE OR REPLACE FUNCTION dpoldoklupdateucto()
  RETURNS trigger AS $$
declare
  _postfix varchar(20);
  _idDokl bigint;
  _zuctovano boolean;
begin  
  select getDoklNamePostfixByModul(NEW.modul) into _postfix;
  if (_postfix = 'Fak') then
    _idDokl := NEW.idDoklFak;
  else
    if (_postfix = 'Int') then
      _idDokl := NEW.idDoklInt;
    else
      if (_postfix = 'Sklad') then
        _idDokl := NEW.idDoklSklad;
      end if;
    end if;
  end if;
  
  execute 'select zuctovano from dDokl' || _postfix || ' where idDokl' || _postfix || ' = ' || _idDokl into _zuctovano;
  
  if (_zuctovano) then
    execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,  
        OLD.idMeny, OLD.idStred, -OLD.sumZkl, -OLD.sumZklMen);  
  
    execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,  
        OLD.idMeny, OLD.idStred, -OLD.sumDph, -OLD.sumDphMen);  
  
    execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,  
        NEW.idMeny, NEW.idStred, NEW.sumZkl, NEW.sumZklMen);  
  
    execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet,  
        NEW.idMeny, NEW.idStred, NEW.sumDph, NEW.sumDphMen);
  end if;  
  
  return NEW;  
end;  
$$ LANGUAGE plpgsql;


-- oprava triggeru ktery uctuje pri mazani polozky
CREATE OR REPLACE FUNCTION dpoldokldeleteucto()
  RETURNS trigger AS $$
declare
  _postfix varchar(20);
  _idDokl bigint;
  _zuctovano boolean;
begin
  select getDoklNamePostfixByModul(OLD.modul) into _postfix;
  if (_postfix = 'Fak') then
    _idDokl := NEW.idDoklFak;
  else
    if (_postfix = 'Int') then
      _idDokl := NEW.idDoklInt;
    else
      if (_postfix = 'Sklad') then
        _idDokl := NEW.idDoklSklad;
      end if;
    end if;
  end if;
  
  execute 'select zuctovano from dDokl' || _postfix || ' where idDokl' || _postfix || ' = ' || _idDokl into _zuctovano;

  if (_zuctovano) then
    execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,  
        OLD.idMeny, OLD.idStred, -OLD.sumZkl, -OLD.sumZklMen);  
  
    execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet,  
        OLD.idMeny, OLD.idStred, -OLD.sumDph, -OLD.sumDphMen);
  end if;
  
  return OLD;  
end;  
$$ LANGUAGE plpgsql;


-- vytvoreni triggeru, ktery zauctuje/oductuje doklad pri zmene priznaku zauctovano
CREATE OR REPLACE FUNCTION ddoklupdateucto()
  RETURNS trigger AS $$
declare
  _postfix varchar(20);
  _idDokl bigint;
  _from varchar(100);
  nasob integer;
begin

  if (NEW.zuctovano <> OLD.zuctovano) then
    select getDoklNamePostfixByModul(NEW.modul) into _postfix;
    if (_postfix = 'Fak') then
      _idDokl := NEW.idDoklFak;
    else
      if (_postfix = 'Int') then
        _idDokl := NEW.idDoklInt;
      else
        if (_postfix = 'Sklad') then
          _idDokl := NEW.idDoklSklad;
        end if;
      end if;
    end if;
    
    _from := 'dPol' || _postfix || ' where idDokl' || _postfix || ' = ' || _idDokl;
  
    if (NEW.zuctovano = true) then
      nasob = 1;
    else
      nasob = -1;
    end if;
  
    execute 'select createOrUpdateObraty(ucetni, storno, datucto, idZklMdUcet, idZklDalUcet, idMeny, idStred, 
                                sumZkl*' || nasob || ', sumZklMen*' || nasob || ') from ' || _from;
                                
    execute 'select createOrUpdateObraty(ucetni, storno, datucto, idDphMdUcet, idDphDalUcet, idMeny, idStred, 
                                sumDph*' || nasob || ', sumDphMen*' || nasob || ') from ' || _from;
  end if;

  return NEW;  
end;  
$$ LANGUAGE plpgsql;
ALTER FUNCTION ddoklupdateucto() OWNER TO %firmaRole%;

-- naveseni na tabulky dokladu, az nakonec radsi proto jmeno zacina dz...
CREATE TRIGGER dzdoklfakupdateucto AFTER UPDATE ON ddoklfak
  FOR EACH ROW EXECUTE PROCEDURE ddoklupdateucto();
CREATE TRIGGER dzdoklintupdateucto AFTER UPDATE ON ddoklint
  FOR EACH ROW EXECUTE PROCEDURE ddoklupdateucto();
CREATE TRIGGER dzdoklskladupdateucto AFTER UPDATE ON ddoklsklad
  FOR EACH ROW EXECUTE PROCEDURE ddoklupdateucto();


-- uprava uctovaci funkce, pokud jsou nejaka vstupni data null konci
-- oprava zjistovani tuzemske meny z aktualniho nastaveni
CREATE OR REPLACE FUNCTION createorupdateobraty(ucetni boolean, storno boolean, datumucto date, castkamducet integer, castkadalucet integer, mena integer, stredisko integer, castka numeric, castkamen numeric)
  RETURNS void AS $$
declare 
  nRok integer; 
  nObdobi integer; 
  idUcetniObdobi integer; 
  idMenyTuzemsko integer; 
  id integer; 
  puvodniMD numeric(15,2); 
  puvodniDal numeric(15,2); 
begin 
  if ucetni = false or storno = true or datumucto is null or stredisko is null or castkamducet is null or castkadalucet is null then 
    return; 
  end if; 
 
  select idMeny into idMenyTuzemsko from nNastav where datumucto >= PlatiOdData order by PlatiOdData desc limit 1; 
  if (idMenyTuzemsko is null) then 
    select idMeny into idMenyTuzemsko from nNastav where (PlatiOdData is null or PlatiOdData <= datumucto)
                                                     and (PlatiDoData is null or PlatiDoData >= datumucto) limit 1;  
  end if; 
 
  nRok:= date_part('year', datumUcto); 
  nObdobi:= date_part('month', datumUcto); 
 
  select idUcetObdobi into idUcetniObdobi from nUcetObdobi where (datumUcto >= PlatiOdData and datumUcto <= PlatiDoData) limit 1; 
 
  /* TODO ma byt podminka na ucetni obdobi"?" */ 
  if(nRok is not null) and (nRok <> 0) and (nObdobi is not null) and (nObdobi <> 0) and (idMenyTuzemsko is not null) and (idUcetniObdobi is not null) then   
     
    /*dph tuzemsko*/ 
    if (castkaMdUcet is not null and stredisko is not null and castka <> 0) then 
      select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty where idUcet = castkaMdUcet and Rok = nRok and Obdobi = nObdobi and 
          idStred = stredisko and idMeny = idMenyTuzemsko; 
 
      if (id is null) then 
        insert into uObraty(idUcet, Rok, Obdobi, idStred, idMeny, ObrDal, ObrMD, idUcetObdobi, LastUpdate)  
	         values(castkaMdUcet, nRok, nObdobi, stredisko, idMenyTuzemsko, 0.0, castka, idUcetniObdobi, now()); 
      else 
        puvodniMD := puvodniMD + castka; 
        if (puvodniMD = 0 and puvodniDal = 0) then 
          delete from uObraty where idObrat = id; 
        else 
          update uObraty set obrMD = puvodniMD, LastUpdate = now() where idObrat = id; 
        end if; 
      end if; 
    end if; 
     
    if (castkaDalUcet is not null and stredisko is not null and castka <> 0) then 
      select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty where idUcet = castkaDalUcet and Rok = nRok and Obdobi = nObdobi and 
          idStred = stredisko and idMeny = idMenyTuzemsko; 
 
      if (id is null) then 
        insert into uObraty (idUcet, Rok, Obdobi, idStred, idMeny, ObrDal, ObrMD, idUcetObdobi, LastUpdate)  
                 values(castkaDalUcet, nRok, nObdobi, stredisko, idMenyTuzemsko, castka, 0.0, idUcetniObdobi, now()); 
      else 
        puvodniDal := puvodniDal + castka; 
        if (puvodniMD = 0 and puvodniDal = 0) then 
          delete from uObraty where idObrat = id; 
        else 
          update uObraty set obrDal = puvodniDal, LastUpdate = now() where idObrat = id; 
        end if; 
      end if; 
    end if; 
 
    -- cizi mena, v pripade, ze se mena nema zapocitavat tak ma hodnotu -1 
    if (mena <> -1) and mena <> idMenyTuzemsko then 
          if(castkaMdUcet is not null and stredisko is not null and castkaMen <> 0) then 
            select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty where idUcet = castkaMdUcet and Rok = nRok and Obdobi = nObdobi and idStred = stredisko and idMeny = mena; 
 
            if (id is null) then 
              insert into uObraty(idUcet, Rok, Obdobi, idStred, idMeny, ObrDal, ObrMD, idUcetObdobi, LastUpdate)  
              values(castkaMdUcet, nRok, nObdobi, stredisko, mena, 0.0, castkaMen, idUcetniObdobi, now()); 
            else 
              puvodniMD := puvodniMD + castkaMen; 
              if (puvodniMD = 0 and puvodniDal = 0) then 
                delete from uObraty where idObrat = id; 
              else 
                update uObraty set obrMD = puvodniMD, LastUpdate = now() where idObrat = id; 
              end if; 
            end if; 
          end if; 
 
          if(castkaDalUcet is not null and stredisko is not null and castkaMen <> 0) then 
            select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty where idUcet = castkaDalUcet and Rok = nRok and Obdobi = nObdobi and idStred = stredisko and idMeny = mena; 
 
            if (id is null) then 
              insert into uObraty(idUcet, Rok, Obdobi, idStred, idMeny, ObrDal, ObrMD, idUcetObdobi, LastUpdate)  
              values(castkaDalUcet, nRok, nObdobi, stredisko, mena, castkaMen, 0.0, idUcetniObdobi, now()); 
            else 
              puvodniDal := puvodniDal + castkaMen; 
              if (puvodniMD = 0 and puvodniDal = 0) then 
                delete from uObraty where idObrat = id; 
              else 
                update uObraty set obrDal = puvodniDal, LastUpdate = now() where idObrat = id; 
              end if; 
            end if; 
          end if; 
    end if; 
 
  else  
    -- v pripade, ze dany objekt nema vyplneno datumUcto, nejsou hodnoty nastaveny a nezapocitavaji se do obratu 
    if (datumUcto is not null) then 
        raise exception 'aktualizace zaznamu tabulky uObraty: Neplatné vstupní hodnoty. % % % %',idMenyTuzemsko,idUcetniObdobi,nRok,nObdobi; 
    end if; 
  end if; 
end; 
$$ LANGUAGE plpgsql;


-- zvyseni cisla verze db
insert into dbversion(iddbversion, dbversion, needreimport)
        values (nextval('seq_dbversion'), 22, 't');