--------------------------------------------------------------------------------
-- Bug 17271 – [WinStrom] Upravit práci se svátkami
--------------------------------------------------------------------------------

-- predelani nekterych poli z integer na numeric
ALTER TABLE nnastav ALTER COLUMN mzdsmenyhodindenne TYPE numeric(6,2);
ALTER TABLE nnastav ALTER COLUMN mzdsmenyhodindenne SET DEFAULT 0.0;
ALTER TABLE nnastav ALTER COLUMN mzdsmenypocpracdoby TYPE numeric(6,2);
ALTER TABLE nnastav ALTER COLUMN mzdsmenypocpracdoby SET DEFAULT 0.0;

-- nastaveni vychozi hodnoty
update nnastav set mzdsmenyhodindenne = 0.0 where mzdsmenyhodindenne is null;
update nnastav set mzdsmenypocpracdoby = 0.0 where mzdsmenypocpracdoby is null;

ALTER TABLE ppracpom ALTER COLUMN uvazhodtydne TYPE numeric(6,2);
ALTER TABLE ppracpom ALTER COLUMN uvazhodtydne SET DEFAULT 0.0;
ALTER TABLE ppracpom ALTER COLUMN zacpracdoby TYPE numeric(6,2);
ALTER TABLE ppracpom ALTER COLUMN zacpracdoby SET DEFAULT 0.0;
ALTER TABLE ppracpom ALTER COLUMN dovolenerocne TYPE numeric(6,2);
ALTER TABLE ppracpom ALTER COLUMN dovolenerocne SET DEFAULT 0.0;

-- nastaveni vychozi hodnoty
update ppracpom set uvazhodtydne = 0.0 where uvazhodtydne is null;
update ppracpom set zacpracdoby = 0.0 where zacpracdoby is null;
update ppracpom set dovolenerocne = 0.0 where dovolenerocne is null;

-- pridani pole, ktere urcuje zpusob vypoctu spatku na pracovni pomer a nastaveni
ALTER TABLE nnastav ADD COLUMN mzdsmenysvatkyprum boolean DEFAULT true;
ALTER TABLE ppracpom ADD COLUMN svatkyprumer boolean DEFAULT true;

-- uprava triggeru aby pracovali korektne s desetinymi cisly
CREATE OR REPLACE FUNCTION pgeneratesmenyforpracpom(_idpracpom integer, _rok integer, _mesic integer)
  RETURNS void AS $$
declare
  _pocatMes date;
  _pracpom record;
  _idsmeny integer;
  _den varchar;
  _mesicS varchar;
  _aktDatum date;
  _denTyden integer;
  _vysledek text;
  _smena integer;
  _smenaSvatek integer;
  _pocatSmeny integer;
  _konecSmeny integer;
  _svatku integer;
begin
  _pocatMes = date(_rok || '-' || _mesic || '-01');
  _mesicS = _mesic;
  if (length(_mesicS) < 2) then
    _mesicS = '0' || _mesicS;
  end if;
  
  select * into _pracpom from ppracpom where idpracpomhlav = _idpracpom and 
    (platiOd is null or platiOd <= _pocatMes) and (platiDo is null or platiDo >= _pocatMes) order by platiOd limit 1;
  
  if (_pracpom is null) then
    return;
  end if;
  
  select idsmeny, planovane into _idsmeny, _vysledek from psmeny where idpracpom = _idpracpom and rok = _rok and mesic = _mesic;
  
  if (_vysledek is null) then
    _vysledek = '';
  end if;
  
  if (_pracpom.pracrozvrhk = 'pracRozvrh.pravidelny') then
    _aktDatum = _pocatMes;
    _smena = 1;
    _smenaSvatek = 1;
    _vysledek = '';
    while extract (month from _aktDatum) = _mesic loop
      _denTyden = extract (dow from _aktDatum);
      _den = extract (day from _aktDatum);
      if (length(_den) < 2) then
        _den = '0' || _den;
      end if;
      if _denTyden = 0 then _denTyden = 7; end if;       -- nedele je 7y den
      
      if (_denTyden <= _pracpom.uvazDnuTydne) then
        select count(*) into _svatku from pmzdyparam where klic = 'svatek' and 
            ((hodnota = _rok || '-' || _mesicS || '-' || _den) or (hodnota = 'XXXX-' || _mesicS || '-' || _den));
      
        if (length(_vysledek) > 0) then _vysledek = _vysledek || ';'; end if;  -- pripojime strednik mezi smeny
        
        _pocatSmeny = (extract (day from _aktDatum) - 1) * 1440 + cast((_pracpom.zacPracDoby * 60) as integer);
        _konecSmeny = _pocatSmeny + cast((60 * _pracpom.uvazHodTydne) as integer) - 1;
        
        if (_svatku = 0) then
          _vysledek = _vysledek || _smena;
          _smena = _smena + 1;
        else  
          _vysledek = _vysledek || 's' || _smenaSvatek;
          _smenaSvatek = _smenaSvatek + 1;
        end if;
        _vysledek = _vysledek || ':' || _pocatSmeny || ':' || _konecSmeny;
      end if;
      
      _aktDatum = _aktDatum + interval '1 day';
    end loop;
  
  end if;
  
  if (_idsmeny is null) then
    insert into psmeny (idosoba, idpracpom, rok, mesic, planovane)
      values (_pracpom.idosoba, _idpracpom, _rok, _mesic, _vysledek);
  else
    update psmeny set planovane = _vysledek where idpracpom = _idpracpom and rok = _rok and mesic = _mesic;
  end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pgeneratesmenyforpracpom(integer, integer, integer) OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION pcreatemzdslozforsmeny(_idSmeny integer)
  RETURNS void AS $$
declare
  SM record;
  SMnext record;
  _hodin numeric;
  _dnuPrac numeric;
  _hodinPoc numeric;
  _dnuPracPoc numeric;
  _konecMes date;
  _okolMes date;
  _denOd integer;
  _denDo integer;
  _denOdPoc integer;
  _denDoPoc integer;
  _pocatek boolean;
  _aktivOd date;
  _aktivDo date;
  i integer;
  smeny varchar[];
  smena varchar[];
  _aktSmena varchar;
  pocSmena integer;
  konSmena integer;
  denSmena integer;
  _idcisMzd integer;
  _idmzdsloz integer;
  _aktHodin numeric;
  _svatek boolean;
  _dnuSvat numeric;
  _hodinSvat numeric;
  _dnuSvatPoc numeric;
  _hodinSvatPoc numeric;
begin
  select * into SM from psmeny where idsmeny = _idsmeny;
  _hodin = 0;
  _dnuPrac = 0;
  _hodinPoc = 0;
  _dnuPracPoc = 0;
  _dnuSvat = 0;
  _hodinSvat = 0;
  _dnuSvatPoc = 0;
  _hodinSvatPoc = 0;
  _konecMes = date(SM.rok || '-' || SM.mesic || '-01') + interval '1 month' - interval '1 day';
  _denOd = 1;
  _denDo = extract (day from _konecMes);
  _denOdPoc = 1;
  _denDoPoc = _denDo;
  
  select aktivniod, aktivnido into _aktivOd, _aktivDo from ppracpomhlav where idpracpomhlav = SM.idpracpom;
  
  if (_aktivOd is not null) then
    if (SM.mesic = extract(month from _aktivOd) and SM.rok = extract(year from _aktivOd)) then
      _denOdPoc = extract (day from _aktivOd);
    end if;
  end if;
  if (_aktivDo is not null) then
    if (SM.mesic = extract(month from _aktivDo) and SM.rok = extract(year from _aktivDo)) then
      _denDoPoc = extract (day from _aktivDo);
    end if;
  end if;
  _pocatek = (_denOd <> _denOdPoc or _denDo <> _denDoPoc);
  
  if (SM.planovane is not null and length(SM.planovane) > 0) then
    smeny = string_to_array(SM.planovane,';');
    _aktSmena = '';
    
    for i in array_lower(smeny,1) .. array_upper(smeny,1) loop  -- prochazime smeny
      smena = string_to_array(smeny[i],':');
      if (smena[1] <> '0' and smena[1] <> 's0') then                     -- zbytek smeny z minuleho mesice nas nezajima
        pocSmena = smena[2];
        konSmena = smena[3];
        
        if (_aktSmena <> smena[1]) then
          denSmena = pocSmena / (24 * 60) + 1;    -- zjistime den
          _svatek = (strpos(smena[1], 's') > 0);    -- zjistime jestli jde o svatek
          
          if (_svatek) then
            _dnuSvat = _dnuSvat + 1;  -- pripocteme svatek kdy mel pracovat ale flakal se
            if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
              _dnuSvatPoc = _dnuSvatPoc + 1;
            end if;
          else
            _dnuPrac = _dnuPrac + 1;  -- pripocteme pracovni den
            if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
              _dnuPracPoc = _dnuPracPoc + 1;
            end if;
          end if;
          _aktSmena = smena[1];
        end if;
        
        _aktHodin = cast((konSmena - pocSmena + 1) as numeric) / 60;
        
        if (_svatek) then
          _hodinSvat = _hodinSvat + _aktHodin;  -- pripocteme svatecni hodiny
          if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
            _hodinSvatPoc = _hodinSvatPoc + _aktHodin;
          end if;
        else
          _hodin = _hodin + _aktHodin;    -- pripocteme pracovni hodiny
          if (denSmena < _denOdPoc or denSmena > _denDoPoc) then
            _hodinPoc = _hodinPoc + _aktHodin;
          end if;
        end if;
      else
        _okolMes = date(SM.rok || '-' || SM.mesic || '-01') - interval '1 month';
        select pcreatemzdslozforsmeny(idsmeny)from psmeny where idpracpom = sm.idpracpom and 
               rok = extract(year from _okolMes) and mesic = extract(month from _okolMes);
      end if;
    end loop;
  end if;
  
  -- podivame se o mesic dal jestli tam nemame kousicek
  /*_okolMes = date(SM.rok || '-' || SM.mesic || '-01') + interval '1 month';
  select * into SMnext from psmeny where idpracpom = sm.idpracpom and 
               rok = extract(year from _okolMes) and mesic = extract(month from _okolMes) limit 1;
  if (SMnext.idsmeny is not null) then
    if (SMnext.planovane is not null and length(SMnext.planovane) > 0) then
      smeny = string_to_array(SMnext.planovane,';');
      
      for i in array_lower(smeny,1) .. array_upper(smeny,1) loop  -- prochazime smeny
        smena = string_to_array(smeny[i],':');
        _aktSmena = smena[1];
      
        if (_aktSmena = '0' or _aktSmena = 's0') then
          pocSmena = smena[2];
          konSmena = smena[3];
          _aktHodin = (konSmena - pocSmena + 1) / 60;
          if (_aktSmena = '0') then
            _hodin = _hodin + _aktHodin;
          else
            _hodinSvat = _hodinSvat + _aktHodin;
          end if;
        end if;
      end loop;  
    end if;
  end if;*/
  
  select idcismzdsloz into _idcisMzd from pcismzdsloz where typslozkyk = 'typSlozky.kopie' and skupslozkyk = 'skupinaSlozky.doba';
  select idmzdsloz into _idmzdsloz from pmzdsloz where idpracpom = SM.idpracpom and rok = SM.rok and mesic = SM.mesic 
      and idcismzdsloz = _idcisMzd;
  
  if (_idmzdsloz is null) then      -- slozka neexistuje vytvorime ji
    select pcreatemzdslozfromcis(_idcisMzd, SM.idpracpom, null, SM.rok, SM.mesic) into _idmzdsloz;
  end if;
  
  update pmzdsloz set dnyKal = (_denDo-_denOd+1), dnyPrac = _dnuPrac, hodiny = _hodin, dnySvatek = _dnuSvat, hodinySvatek = _hodinSvat where idmzdsloz = _idmzdsloz;

  
  select idcismzdsloz into _idcisMzd from pcismzdsloz where typslozkyk = 'typSlozky.inicializace' and skupslozkyk = 'skupinaSlozky.doba';
  if (_idcisMzd is not null) then
    select idmzdsloz into _idmzdsloz from pmzdsloz where idpracpom = SM.idpracpom and rok = SM.rok and mesic = SM.mesic
      and idcismzdsloz = _idcisMzd;

    if (_pocatek) then
      if (_idmzdsloz is null) then      -- slozka neexistuje vytvorime ji
        select pcreatemzdslozfromcis(_idcisMzd, SM.idpracpom, null, SM.rok, SM.mesic) into _idmzdsloz;
      end if;

      update pmzdsloz set dnyKal = (_denDo-_denOd - _denDoPoc+_denOdPoc), dnyPrac = _dnuPracPoc, hodiny = _hodinPoc, dnySvatek = _dnuSvatPoc, hodinySvatek = _hodinSvatPoc where idmzdsloz = _idmzdsloz;
    else
      if (_idmzdsloz is not null) then      -- slozka neexistuje smazeme ji
        delete from pmzdsloz where idmzdsloz = _idmzdsloz;
      end if;
    end if;
  end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pcreatemzdslozforsmeny(integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 230, 't');
