--------------------------------------------------------------------------------
-- Bug 17866 – [WinStrom] 19. Logování změn na jednotlivých záznamech
--------------------------------------------------------------------------------

CREATE SEQUENCE seq_wzurnal
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_wzurnal OWNER TO %admin%;

CREATE TABLE wzurnal (
    idzurnal integer DEFAULT nextval('seq_wzurnal'::regclass) NOT NULL,

    tabulka character varying(255) NOT NULL,
    idzaznamu integer NOT NULL,
    sloupec character varying(255),
    novahod text,
    starahod text,

    op character varying(50),
    transakceK character varying(50),
    uziv character varying(50),
    datcas timestamp without time zone DEFAULT now()::timestamp without time zone NOT NULL,

    CONSTRAINT wzurnal_pkey PRIMARY KEY (idzurnal)
);

ALTER TABLE public.wzurnal OWNER TO %admin%;
GRANT SELECT ON TABLE wzurnal TO %firmaRole%;


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION putglotxvariable(i_kod character varying, i_hodnota character varying)
  RETURNS void AS $$
DECLARE
  tmpGloTxVarTableExists boolean;
  pom integer;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists <> true) THEN
    CREATE TEMP TABLE tmp_gloTxVar (
      kod character varying(50),
      hodnota character varying(50)
    ) WITH (OIDS=FALSE) ON COMMIT DROP;
    ALTER TABLE tmp_gloTxVar OWNER TO %firmaRole%;
  END IF;

  SELECT count(1) INTO pom FROM tmp_gloTxVar WHERE kod=i_kod;
  if (pom > 0) then
    UPDATE tmp_gloTxVar SET hodnota=i_hodnota WHERE kod=i_kod;
  ELSE
    INSERT INTO tmp_gloTxVar VALUES(i_kod, i_hodnota);
  END IF;

END;
$$ LANGUAGE plpgsql;

ALTER FUNCTION putglotxvariable(i_kod character varying, i_hodnota character varying) OWNER TO %firmaRole%;


--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION removeglotxvariable(i_kod character varying)
  RETURNS void AS $$
DECLARE
  tmpGloTxVarTableExists boolean;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists = true) THEN
    DELETE FROM tmp_gloTxVar WHERE kod=i_kod;
  END IF;

END;
$$ LANGUAGE plpgsql;

ALTER FUNCTION removeglotxvariable(i_kod character varying) OWNER TO %firmaRole%;


--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION getglotxvariable(i_kod character varying)
  RETURNS character varying AS $$
DECLARE
  result varchar(50);
  tmpGloTxVarTableExists boolean;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists = true) THEN
    SELECT hodnota INTO result FROM tmp_gloTxVar WHERE kod=i_kod LIMIT 1;
  END IF;

  RETURN result;
END;
$$ LANGUAGE plpgsql;

ALTER FUNCTION getglotxvariable(i_kod character varying) OWNER TO %firmaRole%;


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


CREATE LANGUAGE pltcl;

CREATE OR REPLACE FUNCTION logzurnal() RETURNS trigger AS $$
    # PL/Tcl function body

    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey)
          AND indisprimary"

    if {$TG_op == "INSERT" || $TG_op == "UPDATE"} {
        set pkval [ quote $NEW([ quote $pk ]) ]
    } else {
        set pkval [ quote $OLD([ quote $pk ]) ]
    }

    spi_exec "SELECT now() AS log_time"

    spi_exec "SELECT getGloTxVariable('zurnalTxName') AS tx_name"
    if {![ info exists tx_name ]} {
      set tx_name null
    } else {
      set tx_name '[ quote $tx_name ]'
    }

    if {$TG_op == "UPDATE" || $TG_op == "DELETE"} {

      spi_exec -array C "select attname from pg_attribute where attrelid = $TG_relid and attnum > 0 and not attisdropped" {
          if {$C(attname) == "lastupdate"} {
              continue;
          }

          if {[ info exists NEW([ quote $C(attname) ]) ]} {
              if {[ info exists OLD([ quote $C(attname) ]) ]} {
                  if {$NEW([ quote $C(attname) ]) != $OLD([ quote $C(attname) ])} {
                      spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                          VALUES ('[ quote $log_time ]', session_user,
                              '[ quote $TG_table_name ]',
                              '[ quote $C(attname) ]',
                              '[ quote $pkval ]',
                              '[ quote $NEW([ quote $C(attname) ]) ]',
                              '[ quote $OLD([ quote $C(attname) ]) ]',
                              '[ quote $TG_op ]',
                              $tx_name)"
                  }
              } else {
                  spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                      VALUES ('[ quote $log_time ]', session_user,
                          '[ quote $TG_table_name ]',
                          '[ quote $C(attname) ]',
                          '[ quote $pkval ]',
                          '[ quote $NEW([ quote $C(attname) ]) ]',
                          null,
                          '[ quote $TG_op ]',
                          $tx_name)"
              }
          } else {
              if {[ info exists OLD([ quote $C(attname) ]) ]} {
                  spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                      VALUES ('[ quote $log_time ]', session_user,
                          '[ quote $TG_table_name ]',
                          '[ quote $C(attname) ]',
                          '[ quote $pkval ]',
                          null,
                          '[ quote $OLD([ quote $C(attname) ]) ]',
                          '[ quote $TG_op ]',
                          $tx_name)"
              } else {
                  # no change -> do nothing
              }
          }
      }

    } else {
        if {$TG_op == "INSERT"} {
              spi_exec "INSERT INTO wzurnal(datcas,uziv,tabulka,sloupec,idzaznamu,novahod,starahod,op,transakcek)
                  VALUES ('[ quote $log_time ]', session_user,
                      '[ quote $TG_table_name ]',
                      null,
                      '[ quote $pkval ]',
                      null,
                      null,
                      '[ quote $TG_op ]',
                      $tx_name)"
        }
    }


    if {$TG_op == "DELETE"} {
        return [array get OLD]
    } else {
        return [array get NEW]
    }
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;

ALTER FUNCTION public.logzurnal() OWNER TO %admin%;


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION create_or_drop_zurnal_triggers(docreate boolean)
  RETURNS void AS
$BODY$
BEGIN
  IF docreate THEN
    PERFORM create_or_drop_ucto_zurnal_triggers(true);
    PERFORM create_or_drop_majetek_zurnal_triggers(true);
    PERFORM create_or_drop_mzdy_zurnal_triggers(true);
  ELSE
    PERFORM create_or_drop_ucto_zurnal_triggers(false);
    PERFORM create_or_drop_majetek_zurnal_triggers(false);
    PERFORM create_or_drop_mzdy_zurnal_triggers(false);
  END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;

ALTER FUNCTION create_or_drop_zurnal_triggers(docreate boolean) OWNER TO %admin%;
GRANT EXECUTE ON FUNCTION create_or_drop_zurnal_triggers(docreate boolean) TO %firmaRole%;


CREATE OR REPLACE FUNCTION exist_zurnal_triggers()
  RETURNS boolean AS
$BODY$
DECLARE
  tgcount integer;
BEGIN
  SELECT count(*) INTO tgcount FROM pg_catalog.pg_trigger WHERE tgname LIKE '%zurnal';

  IF (tgcount > 0) THEN
    RETURN true;
  ELSE
    RETURN false;
  END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

ALTER FUNCTION exist_zurnal_triggers() OWNER TO %admin%;
GRANT EXECUTE ON FUNCTION exist_zurnal_triggers() TO %firmaRole%;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 271, 't');
