--------------------------------------------------------------------------------
-- Bug 18577 – [WinStrom] Umoznit nahravani uzivatelskych reportu
--------------------------------------------------------------------------------

alter table wreport add column pocetPriloh integer not null default 0;
alter table wreport add column rozsiritelny boolean default false;
alter table wreport add column sumovany boolean default false;
alter table wreport add column idStatu integer;
alter table wreport add column idTypOrg integer;
alter table wreport add column ordering character varying(255);
alter table wreport add column predvybranyPocetK character varying(50);
alter table wreport add column idPriloha integer;

alter table wpriloha add column idReport integer;

alter table wreport add constraint wpriloha_idreport FOREIGN KEY (idreport)
        REFERENCES wreport(idreport) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX wPriloha_IdReport_index ON wPriloha USING btree (IdReport);
CREATE INDEX wReport_IdPriloha_index ON wReport USING btree (IdPriloha);

create or replace function updatePocetPriloh(prilohaid integer, addon integer) returns void as
$BODY$
declare
    priloha record;
begin
    select * into priloha from wpriloha p where p.idpriloha = prilohaid;

    if (priloha.idreport is not null) then
        update wreport 
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idreport = priloha.idreport)
        where idreport = priloha.idreport;
    end if;


    if (priloha.idadresar is not null) then
        update aadresar 
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idadresar = priloha.idadresar)
        where idfirmy = priloha.idadresar;
    end if;

    if (priloha.idkontakt is not null) then
        update akontakty set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.idkontakt = priloha.idkontakt)
        where idkontakt = priloha.idkontakt;
    end if;

    if (priloha.idcenik is not null) then
        update ccenik set pocetPriloh = (select count(1)+addon
                                        from wpriloha p
                                        where p.idcenik = priloha.idcenik)
        where idcenik = priloha.idcenik;
    end if;

    if (priloha.iddoklfak is not null) then
        update ddoklfak set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklfak = priloha.iddoklfak)
        where iddoklfak = priloha.iddoklfak;
    end if;

    if (priloha.iddoklint is not null) then
        update ddoklint set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklint = priloha.iddoklint)
        where iddoklint = priloha.iddoklint;
    end if;

    if (priloha.iddoklobch is not null) then
        update ddoklobch set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklobch = priloha.iddoklobch)
        where iddoklobch = priloha.iddoklobch;
    end if;

    if (priloha.iddoklsklad is not null) then
        update ddoklsklad set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklsklad = priloha.iddoklsklad)
        where iddoklsklad = priloha.iddoklsklad;
    end if;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION updatePocetPriloh(integer, integer) OWNER TO %firmaRole%;

-- prevedeni stavajicich zaznamu z tabulky report data na prilohy
create or replace function prevedReportDataNaPrilohy() returns void as
$BODY$
declare
  _idprilohadata integer;
  _idPriloha integer;
  _reportData record; 
begin
    for _reportData in select * from wReportData loop
      select nextval('seq_wprilohadata') into _idPrilohaData;
      insert into wprilohadata(idprilohadata, bytedata, blobdata) 
         values (_idPrilohaData, null,_reportData.content);
      select nextval('seq_wpriloha') into _idPriloha;
      insert into wpriloha(idpriloha, nazsoub, contenttype, typK, prilozit, idreport, iddata) 
         values(_idPriloha, _reportData.kod||'.jasper', 'content/unknown', 'typPrilohy.report', true, _reportData.idreport, _idPrilohaData);
      if (_reportData.kod like 'default') then
        update wreport set idpriloha = _idPriloha, predvybranyPocetK = 'predvybranyPocet.aktualni' where idreport = _reportData.idReport;
      end if;
    end loop;
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION prevedReportDataNaPrilohy() OWNER TO %firmaRole%;

begin;
  select prevedReportDataNaPrilohy();
commit;

drop table wreportData;
drop function prevedReportDataNaPrilohy();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 280, 't');
