-------------------------------------------------------------------
-- Bug 19202 – [WinStrom] Rozseknutí tabulky výrobních čísel na dvě
-------------------------------------------------------------------

-- vytvoreni tabulky pro hlavicku vyrobnich cisel
CREATE SEQUENCE seq_svyrobnicislohlav
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_svyrobnicislohlav OWNER TO %firmaRole%;

CREATE TABLE svyrobnicislohlav
(
  idvyrobnicislohlav integer DEFAULT nextval('seq_svyrobnicislohlav'::regclass) NOT NULL,
  kod character varying(255) NOT NULL,
  idCenik integer not null,
  CONSTRAINT svyrobnicislohlav_pkey PRIMARY KEY (idvyrobnicislohlav),
  CONSTRAINT svyrobnicislohlav_idcenik_fk FOREIGN KEY (idCenik)
      REFERENCES ccenik (idcenik)
)
WITH (OIDS=FALSE);
ALTER TABLE svyrobnicislohlav OWNER TO %firmaRole%;

CREATE INDEX sVyrobniCisloHlav_IdCenik_index ON sVyrobniCisloHlav USING btree (IdCenik);

-- pridani noveho sloupce do vyrobnich cisel
ALTER TABLE svyrobnicislo ADD COLUMN idvyrobnicislohlav integer;
ALTER TABLE svyrobnicislo ADD CONSTRAINT idVyrobniCisloHlav_fk FOREIGN KEY (idvyrobnicislohlav)
        REFERENCES svyrobnicislohlav (idvyrobnicislohlav);
ALTER TABLE svyrobnicislo ADD CONSTRAINT sVyrobniCislo_idCenik_fk FOREIGN KEY (idCenik)
        REFERENCES ccenik (idcenik);

CREATE INDEX sVyrobniCislo_IdVyrobniCisloHlav_index ON sVyrobniCislo USING btree (IdVyrobniCisloHlav);

-- presypani dat do nove struktury
CREATE OR REPLACE FUNCTION copyData()
  RETURNS void AS
$BODY$
declare
    vCislo record;
begin
    for vCislo in (select * from svyrobnicislo) loop
        insert into svyrobnicislohlav (kod, idCenik)
            values (vCislo.kod, vCislo.idCenik);

        update svyrobnicislo set idvyrobnicislohlav = currval('seq_svyrobnicislohlav')
            where idvyrobnicislo = vCislo.idvyrobnicislo;
    end loop;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION copyData() OWNER TO %firmaRole%;


select copyData();
drop function copyData();

-- triggery na vytvoreni a smazani hlavicky vyrobniho cisla
CREATE OR REPLACE FUNCTION svyrobnicislocreatehlav() RETURNS trigger AS
$BODY$
declare
  hlav record;
begin
  -- hlavicka neexistuje -> vytvorime ji
  if (NEW.idvyrobnicislohlav is NULL) then
    insert into svyrobnicislohlav(kod, idcenik)
      values (NEW.kod, NEW.idcenik);
    select currval('seq_svyrobnicislohlav') into NEW.idvyrobnicislohlav;
  -- polozka se jiz vaze na hlavicku, pro jistotu zkontrolujeme,
  -- ze mame stejny kod a idcenik
  else
    select * into hlav from svyrobnicislohlav h where h.idvyrobnicislohlav = NEW.idvyrobnicislohlav;
    if (NEW.kod != hlav.kod) then
      raise exception 'Kod pohybu vyrobniho cisla % se neshoduje s kodem na hlavicce %.', NEW.kod, hlav.kod;
    end if;

    if (NEW.idcenik != hlav.idcenik) then
      raise exception 'Cenik pohybu vyrobniho cisla % se neshoduje s cenikem na hlavicce %.', NEW.idcenik, hlav.idcenik;
    end if;
  end if;

  return NEW;
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicislocreatehlav() OWNER TO %firmaRole%;

create trigger svyrobnicislocreatehlav
  before insert or update
  on svyrobnicislo
  for each row
  execute procedure svyrobnicislocreatehlav();

CREATE OR REPLACE FUNCTION svyrobnicislodeletehlav()
  RETURNS trigger AS
$BODY$
declare
  pohybu integer;
begin
  -- spocteni zbyvajicich pohybu k teto hlavicce
  select count(1) into pohybu from svyrobnicislo vc where vc.idvyrobnicislohlav = OLD.idvyrobnicislohlav;

  -- jednalo se o posledni pohyb -> smazeme hlavicku
  if (pohybu = 0) then
    delete from svyrobnicislohlav where idvyrobnicislohlav = OLD.idvyrobnicislohlav;
  end if;

  return OLD;
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicislodeletehlav() OWNER TO %firmaRole%;

create trigger svyrobnicislodeletehlav
  after delete
  on svyrobnicislo
  for each row
  execute procedure svyrobnicislodeletehlav();

-- nastavime novy sloupec NOT NULL
ALTER TABLE svyrobnicislo ALTER COLUMN idvyrobnicislohlav SET NOT NULL;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 298, 't');
