-------------------------------------------------------------------------
-- Bug 20078 – [WinStrom] Opravit zbytecne aktualizace pri zmene dokladu
-------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION comparevarchar(varchara varchar, varcharb varchar)
  RETURNS boolean AS
$BODY$
begin
  if (varchara is null and varcharb is null) or (varchara is not null and varcharb is not null and varchara = varcharb) then
    return true;
  end if;

  return false;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION comparevarchar(varchar, varchar) OWNER TO %firmaRole%;

-- Function: dpolskladupdate()
-- DROP FUNCTION dpolskladupdate();
CREATE OR REPLACE FUNCTION dpolskladupdate() RETURNS trigger AS
$BODY$
declare
  tmp_rozsirenepolozky_exists boolean;
  _updatuj boolean;
  _pocetPolozek integer;
  _rozsirenaPolozka record;
  _sklFifo boolean;
  _vypnutySklad character varying;
  _mazatTmpTable boolean;
begin
  if(NEW.idSkladu = OLD.idSkladu and NEW.idCenik = OLD.idCenik and comparedate(NEW.datVyst, OLD.datVyst)
    and comparevarchar(NEW.typPolozkyK, OLD.typPolozkyK)
    and comparevarchar(NEW.sarze, OLD.sarze) and comparedate(NEW.expirace, OLD.expirace)
    and NEW.mnozMj = OLD.mnozMj and NEW.cenaMj = OLD.cenaMj and NEW.sumZkl = OLD.sumZkl and NEW.sumZklMen = OLD.sumZklMen
  ) then
    return NEW;
  end if;

  /*
    prepocet skladu budeme provadet jen v okamziku kdy neexistuje tabulka tmp_rozsirenePolozky
    nebo kdyz tato tabulka existuje ale neni v ni obsazeno polozka kteoru chceme updatovat
  */
  select getglotxvariable('vypniSklad') into _vypnutySklad;
  if(_vypnutySklad is null or _vypnutySklad <> 'true') then
    _updatuj = true;
    tmp_rozsirenepolozky_exists := (select istableexist('tmp_RozsirenePolozky'));
    if(tmp_rozsirenepolozky_exists = true) then
      select count(*) into _pocetPolozek from tmp_rozsirenePolozky where idPolSklad = NEW.idPolSklad;
      if(_pocetPolozek > 0) then
        _updatuj = false;
      end if;
    end if;

    if(OLD.idPolSklad < 0 and NEW.idPolSklad > 0) then
      _updatuj = false;
    end if;

    _sklfifo := (select isfifosklad(NEW.datvyst));
    if (_sklfifo) then
      --fifo
      if (OLD.storno=false and NEW.storno=true) then
        execute sfifoStorno(OLD.idpolsklad, OLD.iddoklsklad);
      end if;
      if (OLD.cenamj <> NEW.cenamj) then
        execute sfifocenamj(OLD.idpolsklad, OLD.iddoklsklad, NEW.cenamj);
      end if;
      --fifo konec
    end if;

    if(_updatuj) then

      -- generuji rozsirene polozky
      select skladGenerujRozsirenePolozky(NEW.datVyst, NEW.idPolSklad, NEW.idKarty) into _mazatTmpTable;

      execute putglotxvariable('zurnalTxName', 'prepocetSkladu');

      for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop
        if(_rozsirenaPolozka.idPolSklad <> OLD.idPolSklad) then
          execute skladOdpoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad);
        end if;
      end loop;

      if(OLD.storno = false) then
        execute sprepoctiskladkartu(OLD.mnozMj, OLD.cenaMj, OLD.datVyst, OLD.idKarty, OLD.idDoklSklad, OLD.idSkladu, -1);
      end if;

      for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop
        if(_rozsirenaPolozka.idPolSklad <> OLD.idPolSklad) then
          execute skladZapoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad);
        else
          if(NEW.storno = false) then
            execute sprepoctiskladkartu(NEW.mnozMj, NEW.cenaMj, NEW.datVyst, NEW.idKarty, NEW.idDoklSklad, NEW.idSkladu, 1);
          end if;
        end if;
      end loop;

      if(_mazatTmpTable) then
        drop table tmp_RozsirenePolozky;
        drop table tmp_SkladoveKarty;
        execute removeglotxvariable('zurnalTxName');
      end if;

    end if;
  end if;
  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladupdate() OWNER TO %firmaRole%;


-- Function: dpolskladupdatecenamjnakupnafakture()
-- DROP FUNCTION dpolskladupdatecenamjnakupnafakture();
CREATE OR REPLACE FUNCTION dpolskladupdatecenamjnakupnafakture()
  RETURNS trigger AS
$BODY$  
declare 
  _cenaMjNakup numeric(19,6); 
  _idPolFak integer; 
  _modul character varying; 
  _mnozMj numeric(19,6); 
  _oldCenaMjNakup numeric(19,6);
begin 
  select idTabA, modulA into _idPolFak, _modul from dVazebTab where idTabB = NEW.idPolSklad and modulB = 'SKL' and  
     (typVazbyK = 'typVazbyDokl.polozkaSkl' or typVazbyK = 'typVazbyDokl.polozkaSklSada'); 
   
 
  if(_idPolFak is not null and _modul is not null and (_modul = 'FAV' or _modul = 'POK')) then 
    if(_modul = 'FAV') then 
      select mnozMj, cenaMjNakup into _mnozMj, _oldCenaMjNakup from dPolFak where idPolFak = _idPolFak; 
    elsif(_modul = 'POK') then 
      select mnozMj, cenaMjNakup into _mnozMj, _oldCenaMjNakup from dPolInt where idPolInt = _idPolFak; 
    end if; 

    select skladGetCenaMjNakup(_idPolFak, _modul, _mnozMj) into _cenaMjNakup; 

    if(_oldCenaMjNakup = _cenaMjNakup) then
      return NEW;  
    end if;

    execute putglotxvariable('vypniSklad', 'true'); 
    
    if(_modul = 'FAV') then 
      update dpolfak set cenaMjNakup = _cenaMjNakup where idpolfak = _idPolFak; 
    elsif(_modul = 'POK') then 
      update dpolint set cenaMjNakup = _cenaMjNakup where idpolint = _idPolFak; 
    end if; 
    execute removeglotxvariable('vypniSklad'); 
  end if; 
   
  return NEW; 
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladupdatecenamjnakupnafakture() OWNER TO %firmaRole%;

-- Function: skladgenerujrozsirenepolozky(date, integer, integer)
-- DROP FUNCTION skladgenerujrozsirenepolozky(date, integer, integer);
CREATE OR REPLACE FUNCTION skladgenerujrozsirenepolozky(i_datum date, i_idpolsklad integer, i_idskladkarty integer)
  RETURNS boolean AS
$BODY$  
declare  
  tmp_skladovekarty_exists boolean;  
  tmp_rozsirenepolozky_exists boolean;  
  _polSklad record;  
  _skladKarta record;  
  _pocetSkladKarty integer;  
  _result boolean; 
  _idUcetObd integer;
begin  
  -- pokud se zde nebyli tabulky vytvoreny tak se vrati false a pokud ano tak se true 
  _result = false; 
 
  -- vytvoreni temporary tabulky pro ulozeni seznamu skladovych karet na kterych dochazi k prepoctu  
  tmp_skladovekarty_exists := (select istableexist('tmp_SkladoveKarty'));  
  if(tmp_skladovekarty_exists <> true) then  
    --raise exception 'tmp_skladoveKarty se vytvari';  
    CREATE TEMPORARY TABLE tmp_SkladoveKarty (  
      idskladkarty integer  
    ) WITH (OIDS=TRUE) ON COMMIT DROP;  
  end if;  
  
  -- vytvoreni temporary tabulky pro ulozeni seznamu polozek, ktere budou prepocitavat skladove karty  
  tmp_rozsirenepolozky_exists := (select istableexist('tmp_RozsirenePolozky'));  
  if(tmp_rozsirenepolozky_exists <> true) then  
    CREATE TEMPORARY TABLE tmp_RozsirenePolozky (  
      idpolsklad integer,  
      iddoklskladMaster integer,  
      iddoklskladpolozky integer,  
      datum date,  
      skupina integer,  
      typPohybuK character varying,  
      idskladkarty integer,  
      storno boolean  
      --mnozMj numeric(19,6),  
      --cenaMj numeric(19,6),  
      --sumZkl numeric(19,6)  
    ) WITH (OIDS=TRUE) ON COMMIT DROP;  
 
    -- tabulky zde byli vytvoreny tim padem se musi vracet true 
    _result = true; 
  end if;  
    
  if(i_idSkladKarty is not null and i_idSkladKarty <> -1) then  
    -- pokud se jedna o prepocet po vlozeni jedna polozky nebo o rekurzi tak se vse dela jen s jednou skladovou kartou  
    select count(*) into _pocetSkladKarty from tmp_SkladoveKarty where idskladkarty = i_idSkladKarty;  
    if(_pocetSkladKarty = 0) then  
      insert into tmp_SkladoveKarty (idskladkarty) values (i_idSkladKarty);  
      for _polSklad in (select * from dpolsklad where datVyst >= i_datum and idkarty = i_idSkladKarty) loop  
        execute skladInsertRozsirenouPolozku(_polSklad.idPolSklad);  
      end loop;  
    else  
      -- do something stupid - kdyz je skladkarta v seznamu  
    end if;  
  else  
    -- pokud se jedna o celkovy prepocet skladu tak do tmp tabulky skladovych karet dam vsechny pro dane obdobi  
    select iducetobdobi into _idUcetObd from nucetobdobi where platioddata <= i_datum and platidodata >= i_datum; 
    for _skladKarta in (select * from skarty where iducetobdobi = _idUcetObd) loop  
      insert into tmp_SkladoveKarty(idskladkarty) values (_skladKarta.idKarty);  
    end loop;  
  
    for _polSklad in (select * from dpolsklad where datVyst >= i_datum) loop  
      execute skladInsertRozsirenouPolozku(_polSklad.idPolSklad);  
    end loop;  
  end if;  
 
  return _result; 
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION skladgenerujrozsirenepolozky(date, integer, integer) OWNER TO %firmaRole%;

-- Function: dpoldoklupdatesklad()
-- DROP FUNCTION dpoldoklupdatesklad();
CREATE OR REPLACE FUNCTION dpoldoklupdatesklad()
  RETURNS trigger AS
$BODY$
declare
  _idPolDokl integer;
  _idDoklad integer;
  _idPolSklad integer;
  _idDoklSklad integer;
  _dokladGenerovatSkl boolean;
  _typDoklGenerovatSkl boolean;
  _typZasobyK character varying;
  _vyrobniCislo record;
  _typPohybuK character varying;
  _sadaItemCount integer;
  _vypnutySklad character varying;
begin
  select getglotxvariable('vypniSklad') into _vypnutySklad;
  if(_vypnutySklad is null or _vypnutySklad <> 'true') then
    --raise exception 'sem sem se nemel dostat';
    if(NEW.storno <> OLD.storno or NEW.stornoPol <> OLD.stornoPol) then
      return NEW;
    end if;

    if(NEW.modul like 'POK') then
      _idPolDokl = NEW.idPolInt;
      _idDoklad = NEW.idDoklInt;
      select generovatSkl into _dokladGenerovatSkl from dDoklInt where idDoklInt = NEW.idDoklInt;
      select generovatSkl into _typDoklGenerovatSkl from dTypDokl where idTypDokl = (select idTypDokl from dDoklInt where idDoklInt = NEW.idDoklInt);

      select count(*) into _sadaItemCount from csady where idceniksada = (select idcenik from dpolInt where idpolint = NEW.idPolInt);
    elsif (NEW.modul like 'FAV' or NEW.modul like 'FAP') then
      _idPolDokl = NEW.idPolFak;
      _idDoklad = NEW.idDoklFak;
      select generovatSkl into _dokladGenerovatSkl from dDoklFak where idDoklFak = NEW.idDoklFak;
      select generovatSkl into _typDoklGenerovatSkl from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = NEW.idDoklFak);

      select count(*) into _sadaItemCount from csady where idceniksada = (select idcenik from dpolfak where idpolfak = NEW.idPolFak);
    else
      return NEW;
    end if;

    -- pokud se lisi sklad, cenik, typPolozky, mnozMj, sarze, expirace
    -- tak skladovou polozku musim smazat a vytvorit znovu kvuli fifu
    if((NEW.idSkladu <> OLD.idSkladu or NEW.idCenik <> OLD.idCenik
       or NEW.typPolozkyK <> OLD.typPolozkyK or NEW.mnozMj <> OLD.mnozMj
       or NEW.sarze <> OLD.sarze or NEW.expirace <> OLD.expirace)
       and _dokladGenerovatSkl and NEW.zdrojProSkl) then
      --raise exception 'mazu polSklad';
      execute deleteNavazanePolSklad(_idPolDokl, NEW.modul);
    end if;

    if(NEW.typPolozkyK like 'typPolozky.katalog' and _dokladGenerovatSkl and _typDoklGenerovatSkl and NEW.zdrojProSkl) then
      select typZasobyK into _typZasobyK from cCenik where idCenik = NEW.idCenik;

      if (_typZasobyK like 'typZasoby.skladem' or _typZasobyK like 'typZasoby.vyrobek') then
        select findParovanyPolSklad(_idPolDokl, NEW.modul) into _idPolSklad;
        if(_idPolSklad is null) then
          --select createPolSklad(_idPolDokl, NEW.modul) into _idPolSklad;
          if(_sadaItemCount > 0) then
            execute createPolSkladSada(_idPolDokl, NEW.modul);
          else
            select createPolSklad(_idPolDokl, NEW.modul) into _idPolSklad;
          end if;
        else
          if(_sadaItemCount > 0) then
            execute fillPolSkladSada(_idPolDokl, NEW.modul);
          else
            select fillPolSklad(_idPolSklad, _idPolDokl, NEW.modul) into _idPolSklad;
          end if;
        end if;
        -- provedeni upgradu vyrobnich cisel
        if(NEW.modul like 'FAV') then
          for _vyrobniCislo in select * from sVyrobniCislo where idPolFakVydej = NEW.idPolFak and idPolSkladVydej <> _idPolSklad loop
            update sVyrobniCislo set idPolSkladVydej = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo;
          end loop;
        end if;
        if(NEW.modul like 'FAP') then
          for _vyrobniCislo in select * from sVyrobniCislo where idPolFakPrijem = NEW.idPolFak and idPolSkladPrijem <> _idPolSklad loop
            update sVyrobniCislo set idPolSkladPrijem = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo;
          end loop;
        end if;
        if(NEW.modul like 'POK') then
          select typPohybuK into _typPohybuK from dDoklInt where idDoklInt = _idDoklad;
          if(_typPohybuK like 'typPohybu.prijem') then
            for _vyrobniCislo in select * from sVyrobniCislo where idPolIntVydej = NEW.idPolInt and idPolSkladVydej <> _idPolSklad loop
              update sVyrobniCislo set idPolSkladVydej = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo;
            end loop;
          else
            for _vyrobniCislo in select * from sVyrobniCislo where idPolIntPrijem = NEW.idPolInt and idPolSkladPrijem <> _idPolSklad loop
              update sVyrobniCislo set idPolSkladPrijem = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo;
            end loop;
          end if;
        end if;

      end if;
      -- prepocteni sum na dokladu
      select findParovanyDoklSklad(_idDoklad, NEW.modul) into _idDoklSklad;
      execute sprepoctiSumyDokladu(_idDoklSklad, NEW.datVyst);
    end if;
  end if;

  return NEW;
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpoldoklupdatesklad() OWNER TO %firmaRole%;


-- Function: sprepoctisumydokladu(integer, date)
-- DROP FUNCTION sprepoctisumydokladu(integer, date);
CREATE OR REPLACE FUNCTION sprepoctisumydokladu(i_iddoklsklad integer, i_datvyst date)
  RETURNS void AS
$BODY$
declare
 _sumZkl numeric(15,2);
 _sumZklMen numeric(15,2);
 _sumZklDoklad numeric(15,2);
 _sumZklMenDoklad numeric(15,2);
 _datVyst date;
 _datUcto date;
begin
 select sum(sumZkl), sum(sumZklMen) into _sumZkl, _sumZklMen from dPolSklad where idDoklSklad = i_idDoklSklad and stornoPol = false;
 select sumCelkem, sumCelkemMen, datVyst, datUcto into _sumZklDoklad, _sumZklMenDoklad, _datVyst, _datUcto from dDoklSklad where idDoklSklad = i_idDoklSklad;
 if(_sumZkl is null) then _sumZkl = 0.0; end if;
 if(_sumZklMen is null) then _sumZklMen = 0.0; end if;
 if(_sumZkl <> _sumZklDoklad or _sumZklMen <> _sumZklMenDoklad or not compareDate(i_datVyst, _datVyst) or not compareDate(i_datVyst, _datUcto)) then
   update dDoklSklad set lastUpdate = LOCALTIMESTAMP, sumCelkem = _sumZkl, sumCelkemMen = _sumZklMen, datVyst = i_datVyst, datUcto = i_datVyst where idDoklSklad = i_idDoklSklad;
 end if;
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION sprepoctisumydokladu(integer, date) OWNER TO %firmaRole%;

-- Function: sprepoctisumydokladu(integer)
-- DROP FUNCTION sprepoctisumydokladu(integer);
CREATE OR REPLACE FUNCTION sprepoctisumydokladu(i_iddoklsklad integer)
  RETURNS void AS
$BODY$
declare
 _sumZkl numeric(15,2);
 _sumZklMen numeric(15,2);
 _datVyst date;
begin
 --select datVyst into _datVyst from dDoklSklad where idDoklSklad = i_idDoklSklad;
 --execute sprepoctisumydokladu(i_idDoklSklad, _datVyst);
 select sum(sumZkl), sum(sumZklMen) into _sumZkl, _sumZklMen from dPolSklad where idDoklSklad = i_idDoklSklad and stornoPol = false;
 if(_sumZkl is null) then _sumZkl = 0.0; end if;
 if(_sumZklMen is null) then _sumZklMen = 0.0; end if;
 update dDoklSklad set sumCelkem = _sumZkl, sumCelkemMen = _sumZklMen where idDoklSklad = i_idDoklSklad;
end
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION sprepoctisumydokladu(integer) OWNER TO %firmaRole%;


-- Function: compareNumeric(numeric, numeric)
-- DROP FUNCTION compareNumeric(numeric, numeric);
CREATE OR REPLACE FUNCTION compareNumeric(a numeric, b numeric)
  RETURNS boolean AS
$BODY$
begin
  if (a is null and b is null) or (a is not null and b is not null and a = b) then
    return true;
  end if;

  return false;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION compareNumeric(numeric, numeric) OWNER TO %firmaRole%;



-- Function: fillpolsklad(integer, integer, character varying)
-- DROP FUNCTION fillpolsklad(integer, integer, character varying);
CREATE OR REPLACE FUNCTION fillpolsklad(_idpolsklad integer, _idpoldokl integer, _modul character varying)
  RETURNS integer AS
$BODY$
declare
  _polDokl record;
  _polSklad record;
  _idDoklad integer;
  _typPohybuK character varying;
  _typPohybuSklad character varying;
  _cenaMj numeric(19,6);
  _cenaMjTuz numeric(19,6);
  _mnozMj numeric(19,6);
  _sumZkl numeric(19,6);
  _sumZklMen numeric(19,6);
  _idDoklSklad integer;
  _idUcetObdobi integer;
  _idKarty integer;
  _sklad record;
  _kurz numeric(19,6);
  _prumCenaTuz numeric(19,6);
  _isCiziMena boolean;
  _typDoklK character varying;
  _idZklMdUcet integer;
  _idZklDalUcet integer;
begin
  if(_modul like 'POK') then
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl;
    select idDoklInt, typPohybuK into _idDoklad, _typPohybuK from dDoklInt where idDoklInt = _polDokl.idDoklInt;
    _typDoklK = null; -- pro pokladnu dobropisy nejsou
  else
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl;
    select idDoklFak, typPohybuK into _idDoklad, _typPohybuK from dDoklFak where idDoklFak = _polDokl.idDoklFak;
    select typDoklK into _typDoklK from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = _polDokl.idDoklFak);
  end if;

  select findParovanyDoklSklad(_idDoklad, _modul) into _idDoklSklad;

  if(_idDoklSklad is null) then
    select createDoklSklad(_idPolDokl, _idDoklad, _modul) into _idDoklSklad;
  end if;

  select * into _polSklad from dPolSklad where idPolSklad = _idPolSklad;
  select typPohybuK into _typPohybuSklad from dDoklSklad where idDoklSklad = _polSklad.idDoklSklad;

  select findIdUcetniObdobiKDatu(_polDokl.datVyst) into _iducetobdobi;
  select idkarty into _idkarty from skarty where idcenik = _polDokl.idcenik and idtypdokl = _polDokl.idskladu and iducetobdobi = _iducetobdobi;
  select * into _sklad from dTypDokl where idTypDokl = _polDokl.idSkladu;

  select getkurzfromdoklad(_idDoklSklad, _polDokl.datVyst) into _kurz;
  select prumCenaTuz into _prumCenaTuz from sKarty where idKarty = _idKarty;
  select isDoklSkladMenaCizi(_idDoklSklad, _polDokl.datVyst) into _isCiziMena;

  if(_typPohybuSklad like 'typPohybu.vydej') then
    -- vsechny dobropisy generuji prijemky
    --if(_typDoklK like 'typDokladu.dobropis') then
    --  _mnozMj = -_polDokl.mnozMj;
    --else
      _mnozMj = _polDokl.mnozMj;
    --end if;
    -- pokud vydavam ze skladu tak ma byt pouzita prumerna cena
    _cenaMj = _polSklad.cenaMj;
    _sumZkl = _polSklad.sumZkl;
    _sumZklMen = _polSklad.sumZklMen;
    -- nastaveni uctu pro polSklad vydej
    select findZklMdUcetIfVydej(_idPolDokl, _modul) into _idZklMdUcet;
    _idZklDalUcet = _sklad.idPrimUcet;
  else
    if(_typDoklK like 'typDokladu.dobropis' and _modul like 'FAV') then
      -- pokud se jedna o dobropis tak doplnuji nakupni ceny;
      _mnozMj = -_polDokl.mnozMj;
      _cenaMjTuz = _polDokl.cenaMjNakup;
      if(_cenaMjTuz = 0.0) then
        _cenaMjTuz = (select nakupCena from ccenik where idCenik = _polDokl.idCenik);
        if(_cenaMj = 0.0) then
          _cenaMj = _polDokl.cenaMj * _kurz;
        end if;
      end if;
      _cenaMj = _cenaMjTuz / _kurz;
      _sumZkl = _mnozMj * _cenaMjTuz;
      if(_isCiziMena) then
        _sumZklMen = _mnozMj * _cenaMj;
      else
        _sumZklMen = 0.0;
      end if;
    else
      -- pokud na sklad prijmam zkopiruji se ceny z faktury
      _mnozMj = _polDokl.mnozMj;
      _sumZkl = _polDokl.sumZkl;
      if(_mnozMj <> 0.0) then
        _cenaMj = _sumZkl / _mnozMj;
      else
        _cenaMj = _sumZkl;
      end if;
      _cenaMj = _cenaMj / _kurz;
      if(_isCiziMena) then
        if(_mnozMj <> 0.0) then
          _sumZklMen = _cenaMj * _mnozMj;
        else
          _sumZklMen = _cenaMj;
        end if;
      else
        _sumZklMen = 0.0;
      end if;
    end if;
    -- nastaveni uctu pro polSklad prijem
    _idZklMdUcet = _sklad.idPrimUcet;
    _idZklDalUcet = _polDokl.idZklMdUcet;
  end if;


  if(_polSklad.cenaMj <> _cenaMj or _polSklad.sumZkl <> _sumZkl or _polSklad.sumZklMen <> _sumZklMen
    or not compareVarchar(_polSklad.nazev, _polDokl.nazev)
    or not compareVarchar(_polSklad.nazevA, _polDokl.nazevA)
    or not compareVarchar(_polSklad.nazevB, _polDokl.nazevB)
    or not compareVarchar(_polSklad.nazevC, _polDokl.nazevC)
    or not compareDate(_polSklad.datVyst, _polDokl.datVyst)
    or not compareDate(_polSklad.datUcto, _polDokl.datVyst)
    or not compareId(_polSklad.baleniId, _polDokl.baleniId)
    or not compareNumeric(_polSklad.mnozBaleni, _polDokl.mnozBaleni)
    or not compareVarchar(_polSklad.sarze, _polDokl.sarze)
    or not compareDate(_polSklad.expirace, _polDokl.expirace)
    or not compareId(_polSklad.cisRad, _polDokl.cisRad)
    or not compareVarchar(_polSklad.eankod, _polDokl.eanKod)
    or not compareId(_polSklad.idmj, _polDokl.idmj)
    or not compareId(_polSklad.idStred, _polDokl.idStred)
    or not compareId(_polSklad.idZakazky, _polDokl.idZakazky)
    or not compareId(_polSklad.idZklMdUcet, _idZklMdUcet)
    or not compareId(_polSklad.idZklDalUcet, _idZklDalUcet) )
  then
    update dPolSklad set
      lastUpdate = LOCALTIMESTAMP,
      cenaMj = _cenaMj,
      --mnozMj = _mnozMj,
      sumZkl = _sumZkl,
      sumZklMen = _sumZklMen,
      nazev = _polDokl.nazev,
      nazevA = _polDokl.nazevA,
      nazevB = _polDokl.nazevB,
      nazevC = _polDokl.nazevC,
      datVyst = _polDokl.datVyst,
      datUcto = _polDokl.datVyst,
      baleniId = _polDokl.baleniId,
      mnozBaleni = _polDokl.mnozBaleni,
      sarze = _polDokl.sarze,
      expirace = _polDokl.expirace,
      cisRad = _polDokl.cisRad,
      eankod = _polDokl.eanKod,
      idmj = _polDokl.idmj,
      idStred = _polDokl.idStred,
      idMeny = _polSklad.idMeny,
      idZakazky = _polDokl.idZakazky,
      idZklMdUcet = _idZklMdUcet,
      idZklDalUcet = _idZklDalUcet,
      ucetni = true
      where idPolSklad = _idPolSklad;
  end if;

  return _idPolSklad;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION fillpolsklad(integer, integer, character varying) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 349, 't');
