-- Uložená procedura k bug 12655 Našeptávač pro ComboVoField

CREATE OR REPLACE FUNCTION findmatcheditems(i_text character varying, i_table character varying,
  i_pkcolumn character varying, i_kodcolumn character varying, i_nazevcolumn character varying,
  i_condClause character varying, i_startIndex integer, i_pageSize integer)
  RETURNS refcursor AS $$
DECLARE
  _text character varying;
  _condClause character varying;
  _limit integer;
  _nalezeno integer;
  _navratoveHodnoty refcursor; 
BEGIN
  DROP TABLE IF EXISTS matcheditems;
  
  CREATE TEMPORARY TABLE matcheditems (
    id integer PRIMARY KEY,
    kod character varying(255),
    nazev character varying(255),
    txt character varying(255),
    uroven integer
  ) WITH (OIDS=FALSE);

  _text = UPPER(i_text);
  _limit = i_startIndex + i_pageSize;
  
  IF (_condClause IS NOT NULL) THEN
    _condClause = i_condClause || ' AND ';
  ELSE
    _condClause = '';
  END IF;

  -- 1) shoda na kód
  EXECUTE 'INSERT INTO matcheditems(id, kod, nazev, txt, uroven) ' ||
      'SELECT ' || i_pkcolumn || ', ' || i_kodcolumn || ', ' || i_nazevcolumn || ', ' ||
      ' COALESCE(NULLIF(' || i_kodcolumn || ' || '': '' || ' || i_nazevcolumn || ', '': '' || ' || i_nazevcolumn || '), ' || i_nazevcolumn || '), ' ||
      ' 1' ||
      ' FROM ' || i_table ||
      ' WHERE ' || _condClause ||
      ' UPPER(' || i_kodcolumn || ') = ''' || _text || ''' AND ' ||
      i_pkcolumn || ' NOT IN  (SELECT id FROM matcheditems) ORDER BY ' || i_nazevcolumn || ' LIMIT ' || _limit || ';';

  SELECT count(*) INTO _nalezeno FROM matcheditems;
  _limit = i_startIndex + i_pageSize - _nalezeno;

  IF (_limit > 0) THEN
    -- 2) shoda na název
    EXECUTE 'INSERT INTO matcheditems(id, kod, nazev, txt, uroven) ' ||
        'SELECT ' || i_pkcolumn || ', ' || i_kodcolumn || ', ' || i_nazevcolumn || ', ' ||
        ' COALESCE(NULLIF(' || i_kodcolumn || ' || '': '' || ' || i_nazevcolumn || ', '': '' || ' || i_nazevcolumn || '), ' || i_nazevcolumn || '), ' ||
        ' 2' ||
        ' FROM ' || i_table ||
        ' WHERE ' || _condClause ||
        ' UPPER(' || i_nazevcolumn || ') = ''' || _text || ''' AND ' ||
        i_pkcolumn || ' NOT IN  (SELECT id FROM matcheditems) ORDER BY ' || i_nazevcolumn || ' LIMIT ' || _limit || ';';

    SELECT count(*) INTO _nalezeno FROM matcheditems;
    _limit = i_startIndex + i_pageSize - _nalezeno;

    IF (_limit > 0) THEN
      -- 3) shodný prefix kódu nebo názvu
      EXECUTE 'INSERT INTO matcheditems(id, kod, nazev, txt, uroven) ' ||
          'SELECT ' || i_pkcolumn || ', ' || i_kodcolumn || ', ' || i_nazevcolumn || ', ' ||
          ' COALESCE(NULLIF(' || i_kodcolumn || ' || '': '' || ' || i_nazevcolumn || ', '': '' || ' || i_nazevcolumn || '), ' || i_nazevcolumn || '), ' ||
          ' 3' ||
          ' FROM ' || i_table ||
          ' WHERE ' || _condClause ||
          ' (UPPER(' || i_kodcolumn || ') LIKE ''' || _text || '%'' OR UPPER(' || i_nazevcolumn || ') LIKE ''' || _text || '%'')  AND ' ||
          i_pkcolumn || ' NOT IN  (SELECT id FROM matcheditems) ORDER BY ' || i_nazevcolumn || ' LIMIT ' || _limit || ';';

      SELECT count(*) INTO _nalezeno FROM matcheditems;
      _limit = i_startIndex + i_pageSize - _nalezeno;

      IF (_limit > 0) THEN
        -- 4) shodný infix kódu nebo názvu
        EXECUTE 'INSERT INTO matcheditems(id, kod, nazev, txt, uroven) ' ||
            'SELECT ' || i_pkcolumn || ', ' || i_kodcolumn || ', ' || i_nazevcolumn || ', ' ||
            ' COALESCE(NULLIF(' || i_kodcolumn || ' || '': '' || ' || i_nazevcolumn || ', '': '' || ' || i_nazevcolumn || '), ' || i_nazevcolumn || '), ' ||
            ' 4' ||
            ' FROM ' || i_table ||
            ' WHERE ' || _condClause ||
            ' (UPPER(' || i_kodcolumn || ') LIKE ''%' || _text || '%'' OR UPPER(' || i_nazevcolumn || ') LIKE ''%' || _text || '%'')  AND ' ||
            i_pkcolumn || ' NOT IN  (SELECT id FROM matcheditems) ORDER BY ' || i_nazevcolumn || ' LIMIT ' || _limit || ';';
      END IF;
    END IF;
  END IF;

  OPEN _navratoveHodnoty FOR SELECT * FROM matcheditems ORDER BY uroven, nazev LIMIT i_pageSize OFFSET i_startIndex; 
  RETURN _navratoveHodnoty;   
END;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.findmatcheditems(i_text character varying, i_table character varying,
  i_pkcolumn character varying, i_kodcolumn character varying, i_nazevcolumn character varying,
  i_condClause character varying, i_startIndex integer, i_pageSize integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 45, 't');
