--------------------------------------------------------------------------------
-- Bug 24116 – [WinStrom] Automaticke doplneni skladu nevyplni nazev na skladove karte
--------------------------------------------------------------------------------

UPDATE skarty AS k
SET nazev = c.nazev, nazeva = c.nazeva, nazevb = c.nazevb, nazevc = c.nazevc
FROM ccenik AS c
WHERE k.idcenik = c.idcenik AND
  ( NOT ((k.nazev IS NULL AND c.nazev IS NULL) OR (k.nazev IS NOT NULL AND c.nazev IS NOT NULL AND k.nazev = c.nazev)) OR
    NOT ((k.nazeva IS NULL AND c.nazeva IS NULL) OR (k.nazeva IS NOT NULL AND c.nazeva IS NOT NULL AND k.nazeva = c.nazeva)) OR
    NOT ((k.nazevb IS NULL AND c.nazevb IS NULL) OR (k.nazevb IS NOT NULL AND c.nazevb IS NOT NULL AND k.nazevb = c.nazevb)) OR
    NOT ((k.nazevc IS NULL AND c.nazevc IS NULL) OR (k.nazevc IS NOT NULL AND c.nazevc IS NOT NULL AND k.nazevc = c.nazevc)) );


CREATE OR REPLACE FUNCTION skartyinsertzceniku()
  RETURNS trigger AS $$
declare  
  _nazev character varying; 
  _nazeva character varying; 
  _nazevb character varying; 
  _nazevc character varying; 
begin
  IF ( NEW.idcenik IS NOT NULL ) THEN
    SELECT nazev, nazeva, nazevb, nazevc INTO _nazev, _nazeva, _nazevb, _nazevc 
    FROM ccenik
    WHERE idcenik = NEW.idcenik;
    
    NEW.nazev = _nazev;
    NEW.nazeva = _nazeva;
    NEW.nazevb = _nazevb;
    NEW.nazevc = _nazevc;
  END IF;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.skartyinsertzceniku() OWNER TO %firmaRole%;

DROP TRIGGER IF EXISTS skartyinsertzceniku ON skarty;

CREATE TRIGGER skartyinsertzceniku BEFORE INSERT ON skarty
  FOR EACH ROW EXECUTE PROCEDURE skartyinsertzceniku();

--------------------------------------------------------------------------------
-- Bug 24233 – [WinStrom] Umoznit prechod na Postgresql 9.x
--------------------------------------------------------------------------------

-- od postgresql 9.0 neni podporovano prejmenovani paramateru, kvuli moznosti zavleceni chyby 
-- pri volani funkce select deletevazuzel(idzaznamu := 1, tablename := 'Cenik') by nebyl nalezen parametr
DROP FUNCTION deletevazuzel(character varying, integer);

CREATE OR REPLACE FUNCTION deletevazuzel(tablename character varying, _idZaznamu integer)
  RETURNS void AS
$BODY$ 
begin 
    -- tyhle vazebni uzly smazeme, ukazovaly by totiz na smazany zaznam 
    delete from wvazuzel mazvu
        where mazvu.idvazuzel in 
            -- vazebni uzly k tomuhle stromu s tableName a s timto zaznamem tj. ukazuje na idZaznamu 
            (select idvazuzel 
            from wvazuzel vu 
            left join wuzel u on vu.iduzlu = u.iduzel 
            left join wstrom s on u.idstrom = s.idstrom 
            where vu.idzaznamu = _idZaznamu
            and s.tabulka = tableName); 
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION deletevazuzel(character varying, integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 496, 't', %appVersion%, clock_timestamp());
