--------------------------------------------------------------------------------
-- Bug 29788 – [WinStrom] Nový číselník forma dopravy
--------------------------------------------------------------------------------

CREATE SEQUENCE seq_dformadopravy
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_dformadopravy OWNER TO %firmaRole%;

CREATE TABLE dformadopravy
(
  idformadopravy integer DEFAULT nextval('seq_dformadopravy'::regclass) NOT NULL,
  lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
  kod character varying(20) NOT NULL,
  nazev character varying(255) NOT NULL,
  nazeva character varying(255),
  nazevb character varying(255),
  nazevc character varying(255),
  popis text,
  poznam text,
  show boolean NOT NULL DEFAULT true,
  platido integer NOT NULL DEFAULT 9999,
  platiod integer NOT NULL DEFAULT 0,
  uuid character varying(50),
  CONSTRAINT dformadopravy_pkey PRIMARY KEY (idformadopravy)
);
ALTER TABLE dformadopravy OWNER TO %firmaRole%;

-- pridani vazby na typ dokladu a doklady
ALTER TABLE dtypdokl ADD COLUMN idformadopravy integer;
ALTER TABLE dtypdokl ADD CONSTRAINT dtypdokl_idformadopravy FOREIGN KEY (idformadopravy)
  REFERENCES dformadopravy (idformadopravy) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX dTypDokl_IdFormaDopravy_index ON dTypDokl USING btree (IdFormaDopravy);

ALTER TABLE ddoklfak ADD COLUMN idformadopravy integer;
ALTER TABLE ddoklfak ADD CONSTRAINT ddoklfak_idformadopravy FOREIGN KEY (idformadopravy)
  REFERENCES dformadopravy (idformadopravy) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX dDoklFak_IdFormaDopravy_index ON dDoklFak USING btree (IdFormaDopravy);

ALTER TABLE ddoklint ADD COLUMN idformadopravy integer;
ALTER TABLE ddoklint ADD CONSTRAINT ddoklint_idformadopravy FOREIGN KEY (idformadopravy)
  REFERENCES dformadopravy (idformadopravy) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX dDoklInt_IdFormaDopravy_index ON dDoklInt USING btree (IdFormaDopravy);

ALTER TABLE ddoklobch ADD COLUMN idformadopravy integer;
ALTER TABLE ddoklobch ADD CONSTRAINT ddoklobch_idformadopravy FOREIGN KEY (idformadopravy)
  REFERENCES dformadopravy (idformadopravy) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX dDoklObch_IdFormaDopravy_index ON dDoklObch USING btree (IdFormaDopravy);

ALTER TABLE ddoklsklad ADD COLUMN idformadopravy integer;
ALTER TABLE ddoklsklad ADD CONSTRAINT ddoklsklad_idformadopravy FOREIGN KEY (idformadopravy)
  REFERENCES dformadopravy (idformadopravy) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
CREATE INDEX dDoklSklad_IdFormaDopravy_index ON dDoklSklad USING btree (IdFormaDopravy);


create or replace function upgrade() returns void as
$BODY$
declare _idsazby integer;
begin
  -- idstatuleg je konstantni po celou dobu trvani firmy
  if ((select max(idstatuleg) from nnastav) = (select idstatu from astaty where uuid = 'CZ_CZ')) then
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'OSOBNĚ', 'Osobně', true, 0, 9999, 'CZ_OSOBNE');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'ČP', 'Česká pošta', true, 0, 9999, 'CZ_CP');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'IN TIME', 'IN TIME', true, 0, 9999, 'CZ_IN TIME');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'PPL', 'PPL', true, 0, 9999, 'CZ_PPL');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'DHL', 'DHL', true, 0, 9999, 'CZ_DHL');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'PPD', 'PPD', true, 0, 9999, 'CZ_PPD');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'GLS', 'GLS', true, 0, 9999, 'CZ_GLS');
    INSERT INTO dformadopravy (lastupdate, kod, nazev, show, platiod, platido, uuid)
     VALUES ('2012-06-22 00:00:00.000', 'UPS', 'UPS', true, 0, 9999, 'CZ_UPS');
  end if;
end;
$BODY$
LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION upgrade() OWNER TO %firmaRole%;

select upgrade();
drop function upgrade();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 622, 't', %appVersion%, clock_timestamp());
