--------------------------------------------------------------------------------
-- Bug 29969 – [WinStrom] - Objednávky přijaté - realizovatelnost objednávky
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION dgetstavnaskladeaktualni(i_idcenik integer, i_idsklad integer, i_datum date, i_idfirma integer)
  RETURNS numeric AS $$
declare
  _stavMj numeric(19,6); 
  _firmaRezerMj numeric(19,6); 
  _celkemRezerMj numeric(19,6); 
begin
  SELECT dgetstavnaskladeaktualni(i_idcenik, i_idsklad, i_datum) INTO _stavMj; 
  
  -- zjistime kolik je celkem rezervovano - od/do ignorujeme (bug 25844, 25789), muselo by se zohlednovat uz pri plneni pole skladmj a nebo toto pole idealne vyhodit
  select coalesce(sum(skladmj), 0) into _celkemRezerMj from srezervace where idcenik = i_idcenik and idsklad = i_idsklad;
   
  IF (i_idfirma > 0) THEN 
    -- zjistime kolik ma firma rezervovano 
    select coalesce(sum(skladmj), 0) into _firmaRezerMj from srezervace where idfirmy = i_idfirma and idcenik = i_idcenik and idsklad = i_idsklad; 
  ELSE 
    _firmaRezerMj := 0; 
  END IF; 
   
  -- spocteme kolik mu muzeme dat 
  _stavMj := minValue(_stavMj, maxValue(_stavMj - _celkemRezerMj, 0) + _firmaRezerMj); 
   
  RETURN _stavMj; 
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dgetstavnaskladeaktualni(integer, integer, date, integer) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION obchodrealizovatelnost(iddokl integer) RETURNS numeric
    AS $$  
declare 
  zbyvaNakl numeric(15,2); 
  moznoSklad numeric(15,2); 
  moznoNesklad numeric(15,2); 
  vysledek numeric(6,2); 
  _idadresar integer;
begin  
  select coalesce(sum((mnozMj-mnozMjReal) * cenaMj), 0) into zbyvaNakl from dPolObch where idDoklObch = idDokl; 
  
  select idfirmy into _idadresar from ddoklobch where iddoklobch = iddokl;
  
  select coalesce(sum(a.procpol), 0) into moznoSklad from 
   (select minValue( 
        maxValue(dgetstavnaskladeaktualni(P.idCenik, P.idBspSkl, DATE(NOW()), _idadresar), 0),
        sum(mnozMj-mnozMjReal)) 
    * (sum(cenamj * (mnozMj-mnozMjReal)) / sum(mnozMj-mnozMjReal)) as procpol
    from dPolObch as P left outer join ccenik as C on P.idcenik = C.idcenik 
      where idDoklObch = idDokl and C.skladove = true and mnozMj-mnozMjReal > 0
    group by P.idcenik, idbspskl) as a;
 
  select coalesce(sum((mnozMj-mnozMjReal) * cenaMj), 0) into moznoNesklad 
    from dPolObch as P left outer join ccenik as C on P.idcenik = C.idcenik  where idDoklObch = idDokl and (C.skladove is null or C.skladove = false); 
 
  if (zbyvaNakl = 0) then 
    vysledek := 100 ; 
  else 
    vysledek := ((moznoSklad + moznoNesklad) / zbyvaNakl) * 100; 
  end if; 
   
  return vysledek; 
end;  
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.obchodrealizovatelnost(iddokl integer) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION drefreshrealizovatelnost() RETURNS void AS $$ 
declare 
begin
  UPDATE ddoklobch SET procReal=obchodrealizovatelnost(idDoklObch) WHERE 
    modul = 'OBP' and sumcelkem > 0 and (stavuzivk is null or stavuzivk not in ('stavDoklObch.hotovo', 'stavDoklObch.vydano' ));
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.drefreshrealizovatelnost() OWNER TO %firmaRole%;

DROP TRIGGER dpolobchrealizovatelnost ON dpolobch;
DROP FUNCTION dpolobchrealizovatelnost();

DROP TRIGGER ddoklobchrealizovatelnost ON ddoklobch;
DROP FUNCTION ddoklobchrealizovatelnost();

DROP TRIGGER dpolskladupdaterealizovatelnost ON dpolsklad;
DROP FUNCTION dpolskladupdaterealizovatelnost();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 626, 't', %appVersion%, clock_timestamp());