-- Function: sprepoctisklad(integer)
-- DROP FUNCTION sprepoctisklad(integer);
CREATE OR REPLACE FUNCTION sprepoctisklad(i_ucetobd integer)
  RETURNS void AS
$BODY$    
/* Prepocte stavy na skladove karte s id = iSkladKartaId */    
declare    
  /* Deklarace promennych */    
  _zaznam Record;
  _skladKarta Record;
  _dPolSkladPomoc Record;
  _pocatek_ucetobd Date;
  _konec_ucetobd Date;
  _typPohybuk character varying(255);
  _sklfifo boolean;
  _prumCenaTuz numeric(19,6);
  tmp_polsklad_exists boolean;
  
begin
  -- vytvoreni temporary tabulky na vypnuti triggeru
  tmp_polsklad_exists := (select istableexist('tmp_polsklad'));
  if(tmp_polsklad_exists <> true) then
    CREATE TEMPORARY TABLE tmp_polsklad (
      idpolsklad integer
    ) WITH (OIDS=TRUE);
  end if;

  -- nacteni datumu zacatku ucetniho obdobi
  select platioddata, platidodata into _pocatek_ucetobd, _konec_ucetobd from nucetobdobi where iducetobdobi = i_ucetobd;

  -- smazani tabulky fifo vazeb
  delete from sfifovazby where idpolskladvydej in 
    (select idpolsklad from dpolsklad where 
            datVyst >= _pocatek_ucetobd and datVyst <= _konec_ucetobd);

  -- nastaveni skladove karty na den zacatku ucetniho obdobi
  for _skladKarta in select * from skarty where iducetobdobi = i_ucetobd loop
    _prumCenaTuz := 0;
    if(_skladKarta.pocatMj <> 0.0) then
      _prumCenaTuz := _skladKarta.pocatTuz / _skladKarta.pocatMj;
    end if;
    update skarty set 
      prumcenatuz = _prumCenaTuz, 
      stavmj = _skladKarta.pocatMj, 
      stavtuz = _skladKarta.pocatTuz,
      stavmen = _skladKarta.pocatMen
      where idkarty = _skladKarta.idKarty;
  end loop;

  -- vypnuti triggeru
  --alter table dpolsklad disable trigger all;
  --alter table dpolsklad enable trigger dpolskladupdatezauctovano;
  alter table dpolsklad disable trigger dpolskladinsert;
  alter table dpolsklad disable trigger dpolskladupdate;
  alter table dpolsklad disable trigger dpolskladdelete;

  -- prepocet skladu
  for _zaznam in select * from dpolsklad 
    where datVyst >= _pocatek_ucetobd and datVyst <= _konec_ucetobd
    order by DatVyst, idPolSklad asc loop
    select typpohybuk into _typPohybuk from ddoklsklad where iddoklsklad = _zaznam.iddoklsklad; 
    --pokud výdej, tak fifuju vazby 
    if (_typPohybuk = 'typPohybu.vydej') then 
      execute sfifoVydej(_zaznam.idPolSklad, _zaznam.idCenik, _zaznam.idSkladu, _zaznam.idKarty, _zaznam.mnozmj); 
      execute sPrepoctiSumyDokladu(_zaznam.idDoklSklad); 
    end if;          

    -- prumerne ceny 
    _sklfifo := (select isfifosklad(_zaznam.datvyst));
    if (_sklfifo <> true) then  
      execute sZmenCenyNaPolSklad(_zaznam.idPolSklad, _zaznam.idkarty, _zaznam.iddoklsklad);    
    end if;

    select * into _dPolSkladPomoc from dpolsklad where idpolsklad = _zaznam.idpolsklad;
    
    execute sPrepoctiSkladKartu(_dPolSkladPomoc.MnozMJ,_dPolSkladPomoc.CenaMJ,_dPolSkladPomoc.DatVyst,_dPolSkladPomoc.idKarty,_dPolSkladPomoc.idDoklSklad,_dPolSkladPomoc.idskladu,1);
  end loop; 

  -- zapnuti triggeru
  alter table dpolsklad enable trigger dpolskladinsert;
  alter table dpolsklad enable trigger dpolskladupdate;
  alter table dpolsklad enable trigger dpolskladdelete;
end;    
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION sprepoctisklad(integer) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Function: sprepoctiskladkartu(numeric, numeric, date, integer, integer, integer, integer)
-- DROP FUNCTION sprepoctiskladkartu(numeric, numeric, date, integer, integer, integer, integer);
CREATE OR REPLACE FUNCTION sprepoctiskladkartu(i_mnozmj numeric, i_cenamj numeric, i_datvyst date, i_skladkartaid integer, i_doklskladid integer, i_skladid integer, i_nasobic integer)
  RETURNS void AS
$BODY$    
/* Prepocte stavy na skladove karte s id = iSkladKartaId */    
declare    
  /* Deklarace promennych */    
  _PomMnozMJ numeric(19,2);    
  _StavMJ numeric(19,2);    
  _StavTuz numeric(19,2);    
  _StavMen numeric(19,2);    
  _PrumCenaTuz numeric(19,2);    
  _PrumCenaMen numeric(19,2);    
  _PoslCenaTuz numeric(19,2);    
  _PoslCenaMen numeric(19,2);    
  _TypPohybuK varchar(50);    
  _DatPosl date;    
  _kurzSkladKarty numeric(19,6);   
  _kurzDokladu numeric(19,6);   
  loc_MnozMJ numeric(19,6);   
  _sklzapor boolean;  
  _sklfifo boolean;
    
  /* Deklarace kurzoru pro nacteni potrebnych dat ze skladove karty a dokladu */    
  declare ThisDoklSklad cursor for select TypPohybuK from dDoklSklad where idDoklSklad = i_DoklSkladId;    
  declare ThisSkladKarta cursor for select StavMJ,StavTuz,StavMen,PrumCenaTuz,PrumCenaMen,DatPosl from sKarty where idKarty = i_SkladKartaId;      
begin    
  loc_MnozMJ:= i_MnozMJ;    
    
  /* Nacteni dat ze skladove karty */    
  open ThisSkladKarta;    
  fetch ThisSkladKarta into _StavMJ,    
    _StavTuz,_StavMen,_PrumCenaTuz,_PrumCenaMen,_DatPosl;    
  close ThisSkladKarta;    
      
  /* Zjisteni typPohybuK */    
  open ThisDoklSklad;    
  fetch ThisDoklSklad into _TypPohybuK;    
  close ThisDoklSklad;    
   
  /* zjisteni meny skladove karty */   
  _KurzSkladKarty := (select getKurzFromSklad(i_skladId, i_DatVyst));   
  _KurzDokladu := (select getKurzFromDoklad(i_doklSkladId, i_DatVyst));   
     
  /* pokud jde o vydej musim odecitat */    
  if _TypPohybuK = 'typPohybu.vydej' then    
    loc_MnozMJ:= 0-loc_MnozMJ;    
  end if;    
      
  /* Vypocet StavMJ */    
  _StavMJ:=_StavMJ+loc_MnozMJ*i_Nasobic;    
   
  if _StavMJ < 0 then   
    select sklzapor into _sklzapor from nNastav where i_DatVyst >= PlatiOdData order by PlatiOdData desc limit 1;   
	if (_sklzapor is null) then   
		select sklzapor into _sklzapor from nNastav where PlatiOdData is null limit 1;    
	end if;   
    if _sklzapor is null or _sklzapor = false then   
        --raise exception '0002:sPreporctiSkladKartu: neni povolen zaporny stav skladu';
    end if;   
  end if;   
	   
      
  /* Vypocet StavTuz */    
  if loc_MnozMJ = 0 then    
    _PomMnozMJ:=1;    
  else    
    _PomMnozMJ:=loc_MnozMJ;    
  end if;    
  _StavTuz:=_StavTuz+(_PomMnozMJ*i_CenaMJ*i_Nasobic*_KurzDokladu); 
  /* Vypocet StavMen */    
  _StavMen:=_StavMen+(_PomMnozMJ*i_CenaMJ*i_Nasobic*_KurzDokladu)/_KurzSkladKarty;    
   
  /* Vypocet prumernych cen ma byt proveden jen pri prijmu*/   
  if _TypPohybuK = 'typPohybu.prijem' then 
    /* Vypocet PrumCenaTuz */    
    if _StavMJ > 0 then    
      _PrumCenaTuz:=_StavTuz/_StavMJ;    
    end if;    
    /* Vypocet PrumCenaMen */    
    if _StavMJ > 0 then    
      _PrumCenaMen:=_StavMen/_StavMJ;    
    end if;    
  end if;    
    
  /* Prepocteni stavu skladu pro FIFO */ 
  -- drobna uprava prumerne ceny pokud by doslo k chybe vlivem zaokrouhleni a pro fifo nutnost   
  _sklfifo := (select isfifosklad(i_DatVyst));   
  --if (_sklfifo) then   
    if(_StavMJ <> 0.0) then  
      _PrumCenaTuz := _StavTuz / _StavMJ;   
      _PrumCenaMen := _StavMen / _StavMJ;  
    end if;  
  --end if;   
    
  /* Nastaveni posledni ceny */    
  _PoslCenaTuz:= i_CenaMJ * _KurzDokladu;   
  _PoslCenaMen:= i_CenaMJ;    
   
  /* Nastaveni data posledniho pohybu na karte */    
  if _DatPosl is null then    
    _DatPosl:= i_DatVyst;    
  end if;    
  if _DatPosl < i_DatVyst then    
    _DatPosl:=i_DatVyst;    
  end if;    
   
  /* Update v databazi */ 
  update sKarty set StavMJ = _StavMJ,    
    StavTuz = _StavTuz,    
    StavMen = _StavMen,    
    PrumCenaTuz = _PrumCenaTuz,    
    PrumCenaMen = _PrumCenaMen,    
    PoslCenaTuz = _PoslCenaTuz,    
    PoslCenaMen = _PoslcenaMen,    
    DatPosl = _DatPosl where    
    idKarty = i_SkladKartaId;    
end;    
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION sprepoctiskladkartu(numeric, numeric, date, integer, integer, integer, integer) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Function: szmencenynapolsklad(integer, integer, integer)
-- DROP FUNCTION szmencenynapolsklad(integer, integer, integer);
CREATE OR REPLACE FUNCTION szmencenynapolsklad(i_polskladid integer, i_skladkartaid integer, i_doklskladid integer)
  RETURNS void AS
$BODY$    
declare    
  /* Deklarace promennych */    
  _MnozMJ numeric(19,2);   
  _CenaMJOld numeric(19,2);    
  _SumZkl numeric(19,2);    
  _SumZklMen numeric(19,2);    
  _PrumCenaTuz numeric(19,2);    
  _PrumCenaMen numeric(19,2);   
  _PrumCena numeric(19,2);    
  _TypPohybuK varchar(50);    
  _DatVyst date;   
  _CiziMena boolean;     
  zaznam record;   
  _KurzSkladKarty numeric(19,6);    
  _KurzDokladu numeric(19,6); 
  tmp_polsklad_exists boolean;
      
  /* Deklarace kurzoru pro nacteni potrebnych dat ze skladove karty a dokladu */    
  declare ThisDoklSklad cursor for select TypPohybuK, DatVyst from dDoklSklad where idDoklSklad = i_DoklSkladId;   
  declare ThisPolSklad cursor for select MnozMJ, CenaMJ from dPolSklad  where idPolSklad = i_PolSkladId;   
  declare ThisSkladKarta cursor for select PrumCenaTuz,PrumCenaMen from sKarty where idKarty = i_SkladKartaId;   
  declare ThisPolSkladSum cursor for select SumZkl, SumZklMen from dPolSklad where idDoklSklad = i_DoklSkladId;      
begin   
  /* Zjisteni typPohybuK */   
  open ThisDoklSklad;    
  fetch ThisDoklSklad into _TypPohybuK, _DatVyst;    
  close ThisDoklSklad;    
   
  /* Zjisteni mnozstvi jednotek */   
  open ThisPolSklad;    
  fetch ThisPolSklad into _MnozMJ, _CenaMJOld;    
  close ThisPolSklad;    
   
  /* Nacteni dat ze skladove karty */   
  open ThisSkladKarta;    
  fetch ThisSkladKarta into _PrumCenaTuz,_PrumCenaMen;    
  close ThisSkladKarta;   
     
  if _TypPohybuK = 'typPohybu.vydej' then   
    --_KurzSkladKarty := (select getKurzFromSklad(i_skladId, i_DatVyst));    
    _KurzDokladu := (select getKurzFromDoklad(i_DoklSkladId, _DatVyst));    
    _PrumCena := _PrumCenaTuz / _KurzDokladu; 
    --_CiziMena := (select isDoklSkladMenaCizi(i_DoklSkladId, _DatVyst));    
    --if _CiziMena then    
    --  _PrumCena := _PrumCenaMen;    
    --else    
    --  _PrumCena := _PrumCenaTuz;    
    --end if;    
 
    _SumZkl := _PrumCenaTuz * _MnozMJ; 
    _SumZklMen := 0; 
    if(_KurzDokladu <> 1.0) then 
      _sumZklMen := _PrumCena * _MnozMJ; 
    end if; 
    tmp_polsklad_exists := (select istableexist('tmp_polsklad'));
    if(tmp_polsklad_exists) then
      insert into tmp_polsklad(idpolsklad) values (i_polskladid);
      update dPolSklad set    
        cenaMJ = _PrumCena,   
        sumZkl = _SumZkl, 
        sumZklMen = _SumZklMen 
        where    
        idPolSklad = i_PolSkladId;   
    end if;
    /* Nacteni novych souctu na skladovem dokladu */   
    _SumZkl := 0;   
    _SumZklMen := 0;    
    for zaznam in select * from dPolSklad where idDoklSklad = i_DoklSkladId loop    
      _SumZkl := _SumZkl + zaznam.sumZkl;   
      _SumZklMen := _SumZklMen + zaznam.sumZklMen;   
    end loop;
  
    update dDoklSklad set   
      sumCelkem = _SumZkl,   
      sumCelkemMen = _SumZklMen   
      where   
      idDoklSklad = i_DoklSkladId;   
   
  end if;   
end;   
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION szmencenynapolsklad(integer, integer, integer) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Function: dpolskladupdate()
-- DROP FUNCTION dpolskladupdate();
CREATE OR REPLACE FUNCTION dpolskladupdate()
  RETURNS trigger AS
$BODY$   
declare  
  _sklfifo boolean; 
  tmp_polsklad_exists boolean;
  _polsklad_count integer;
begin
  -- vytvoreni temporary tabulky na vypnuti triggeru
  tmp_polsklad_exists := (select istableexist('tmp_polsklad'));
  if(tmp_polsklad_exists <> true) then
    --raise exception 'tmp_polSklad se vytvari';
    CREATE TEMPORARY TABLE tmp_polsklad (
      idpolsklad integer
    ) WITH (OIDS=TRUE);
  end if;

  select count(*) into _polsklad_count from tmp_polsklad where idpolsklad = NEW.idpolsklad;
  if(_polsklad_count > 0) then
    delete from tmp_polsklad where idpolsklad = NEW.idpolsklad;
  else  
    _sklfifo := (select isfifosklad(NEW.datvyst)); 
    if (_sklfifo) then 
      --fifo 
      if (OLD.storno=false and NEW.storno=true) then 
	execute sfifoStorno(OLD.idpolsklad, OLD.iddoklsklad); 
      end if;   
      if (OLD.cenamj <> NEW.cenamj) then 
	execute sfifocenamj(OLD.idpolsklad, OLD.iddoklsklad, NEW.cenamj); 
      end if;   
      --fifo konec 
    end if; 

    -- prum ceny 
    if OLD.storno = false then  
      execute sZapoctiDoMinulosti(OLD.MnozMJ,OLD.CenaMJ,OLD.DatVyst,OLD.idKarty,OLD.idDoklSklad,OLD.idSkladu,-1, OLD.idPolSklad);   
    end if;  
    if NEW.storno = false then  
      execute sZapoctiDoMinulosti(NEW.MnozMJ,NEW.CenaMJ,NEW.DatVyst,NEW.idKarty,NEW.idDoklSklad,NEW.idSkladu,1, NEW.idPolSklad);   
    end if;  
    -- prum ceny konec 
  end if;
  return NEW;   
end;   
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladupdate() OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Trigger: dpolskladupdate on dpolsklad
DROP TRIGGER dpolskladupdate ON dpolsklad;
CREATE TRIGGER dpolskladupdate
  AFTER UPDATE
  ON dpolsklad
  FOR EACH ROW
  EXECUTE PROCEDURE dpolskladupdate();

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Function: dpolskladinsert()
-- DROP FUNCTION dpolskladinsert();
CREATE OR REPLACE FUNCTION dpolskladinsert()
  RETURNS trigger AS
$BODY$   
declare   
  _typPohybuk character varying(255); 
  _sklfifo boolean; 
  tmp_polsklad_exists boolean;
  _polsklad_count integer;
begin
  -- vytvoreni temporary tabulky na vypnuti triggeru
  tmp_polsklad_exists := (select istableexist('tmp_polsklad'));
  if(tmp_polsklad_exists <> true) then
    --raise exception 'tmp_polSklad se vytvari';
    CREATE TEMPORARY TABLE tmp_polsklad (
      idpolsklad integer
    ) WITH (OIDS=TRUE);
  end if;

  select count(*) into _polsklad_count from tmp_polsklad where idpolsklad = NEW.idpolsklad;
  if(_polsklad_count > 0) then
    delete from tmp_polsklad where idpolsklad = NEW.idpolsklad;
  else    
    --_sklfifo := (select isfifosklad(NEW.datvyst)); 
    --if (_sklfifo) then 
        --pokud výdej, tak fifuju vazby 
	select typpohybuk into _typPohybuk from ddoklsklad where iddoklsklad = NEW.iddoklsklad; 
	--pokud výdej, tak fifuju vazby 
	if (_typPohybuk = 'typPohybu.vydej') then 
		execute sfifoVydej(NEW.idPolSklad, NEW.idCenik, NEW.idSkladu, NEW.idKarty, NEW.mnozmj); 
        execute sPrepoctiSumyDokladu(NEW.idDoklSklad); 
	end if;          
    --end if; 
   
    -- prumerne ceny 
    execute sZapoctiDoMinulosti(NEW.MnozMJ,NEW.CenaMJ,NEW.DatVyst,NEW.idKarty,NEW.idDoklSklad,NEW.idSkladu,1, NEW.idPolSklad);       
    --prumerne ceny konec   
  end if;
  return NEW;   
end;   
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladinsert() OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Function: istableexist(varchar)
-- DROP FUNCTION istableexist(varchar);
CREATE OR REPLACE FUNCTION istableexist(tablename varchar) RETURNS boolean AS 
$BODY$
DECLARE
BEGIN
  /* check the table exist in database and is visible*/
  PERFORM relname,relnamespace FROM pg_class
    WHERE relkind = 'r'
    AND Upper(relname) = Upper(tablename)
    AND pg_table_is_visible(oid);

  IF FOUND THEN
    RETURN TRUE;
  ELSE
    RETURN FALSE;
  END IF;
END;
$BODY$
LANGUAGE 'plpgsql';
ALTER FUNCTION istableexist(varchar) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- Function: szapoctipolozkyoddata(integer, date, integer)
-- DROP FUNCTION szapoctipolozkyoddata(integer, date, integer);
CREATE OR REPLACE FUNCTION szapoctipolozkyoddata(i_skladkartaid integer, i_datum date, i_polskladid integer)
  RETURNS void AS
$BODY$    
declare    
  zaznam Record;
  _polskladpomoc Record;
  _sklfifo boolean;  
  _idPolSklad integer; 
begin    
  _idPolSklad := i_polskladid; 
  if(_idPolSklad < 0) then 
    _idPolSklad := -1*_idPolSklad; 
  end if;    
  for zaznam in select * from dPolSklad where ((DatVyst > i_Datum) or (DatVyst = i_Datum and idPolSklad > _idPolSklad))   
  and idKarty = i_SkladKartaId order by DatVyst, idPolSklad asc LOOP   
    _sklfifo := (select isfifosklad(zaznam.datvyst));  
    if (_sklfifo <> true) then  
      execute sZmenCenyNaPolSklad(zaznam.idPolSklad, i_SkladKartaId, zaznam.iddoklsklad);    
    end if;

    select * into _polskladpomoc from dpolsklad where idpolsklad = zaznam.idpolsklad;
    
    execute sPrepoctiSkladKartu(_polskladpomoc.MnozMJ,_polskladpomoc.CenaMj,_polskladpomoc.DatVyst,_polskladpomoc.idKarty,_polskladpomoc.idDoklSklad,_polskladpomoc.idSkladu,1);    
  end loop;    
end;    
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION szapoctipolozkyoddata(integer, date, integer) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 67, 't');
