--------------------------------------------------------------------------------
-- Bug 37660 – [FlexiBee] Nejednoznačné mapování změn v changes API
--------------------------------------------------------------------------------

ALTER TABLE changelog ADD COLUMN evidence varchar(127);
CREATE INDEX changelog_evidence_index ON changelog USING btree (evidence);


CREATE OR REPLACE FUNCTION logchanges() RETURNS trigger AS $$
    spi_exec "SELECT pg_attribute.attname AS pk
        FROM pg_index, pg_class, pg_attribute
        WHERE
          pg_class.oid = '[ quote $TG_table_name ]'::regclass AND
          indrelid = pg_class.oid AND
          pg_attribute.attrelid = pg_class.oid AND
          pg_attribute.attnum = any(pg_index.indkey)
          AND indisprimary"

    set modul null
    if {$TG_op == "INSERT" || $TG_op == "UPDATE"} {
        set pkval [ quote $NEW([ quote $pk ]) ]
        if {[info exists NEW(modul)]} {
            set modul '[ quote $NEW(modul) ]'
        }
    } else {
        set pkval [ quote $OLD([ quote $pk ]) ]
        if {[info exists OLD(modul)]} {
            set modul '[ quote $OLD(modul) ]'
        }
    }

    spi_exec "SELECT session_user AS user_name"
    set user_dollar_idx [string first \$ $user_name] 
    if {$user_dollar_idx != -1} {
        incr user_dollar_idx
        set user_name "[string range $user_name $user_dollar_idx end]" 
    }

    set operation "[string range $TG_op 0 0]"
    set extid null
    set extid_table "wextid"

    if {$TG_op == "DELETE" && $TG_when == "AFTER"} {
        set extid_table "changelog_extid_${TG_table_name}_pk${pkval}"
        if {[spi_exec "SELECT 1 FROM pg_tables WHERE tablename = '$extid_table'"] == 0} {
            elog WARNING "Temporary table '$extid_table' was not found. External identifiers (extid) will be missed in Changes API for this DELETE." 
            set extid_table "wextid"
        }
    }

    if {[ info exists NEW(kod) ]} {
        append all_extids 'code:[ quote $NEW(kod) ]'
    } elseif {[ info exists OLD(kod) ]} {
        append all_extids 'code:[ quote $OLD(kod) ]'
    }

    spi_exec "select tablename from pg_tables where tablename = '${extid_table}'" {
        spi_exec "select attname AS fk from pg_attribute where attrelid = 'wextid'::regclass and attnum in (select conkey\[1\] from pg_constraint where conrelid = 'wextid'::regclass and confrelid = '[ quote $TG_table_name ]'::regclass)" {
            spi_exec "select extid from $extid_table where $fk = $pkval" {
                if {[info exists all_extids] && [string length $all_extids] > 0} {
                  append all_extids ","
                }
                append all_extids '[ quote $extid ]'
            }
        }
    }

    if {[info exists all_extids] && [string length $all_extids] > 0} {
        set extid "array\[$all_extids\]"
    }

    array set evidence_discriminators {
        dsmlouvy+null typpohybuk
        dtypsml+null typpohybuk
        
        ddoklint+BAN zapocet
        dtypdokl+BAN zapocet
        
        ddoklfak+FAV prodejka
        dtypdokl+FAV prodejka
        
        ukurzy+null typkurzuk
        
        sumisteni+null typumistenik
    }
    
    array set evidence_disc_mapping {
        dsmlouvy+null:typPohybu.prijem smlouva
        dsmlouvy+null:typPohybu.vydej dodavatelska-smlouva
        
        dtypsml+null:typPohybu.prijem typ-smlouvy
        dtypsml+null:typPohybu.vydej dodavatelsky-typ-smlouvy
        
        ddoklint+BAN:t vzajemny-zapocet
        ddoklint+BAN:f banka
        
        dtypdokl+BAN:t typ-vzajemnych-zapoctu
        dtypdokl+BAN:f typ-banka
        
        ddoklfak+FAV:t prodejka
        ddoklfak+FAV:f faktura-vydana
        
        dtypdokl+FAV:t typ-prodejky
        dtypdokl+FAV:f typ-faktury-vydane
        
        ukurzy+null:typKurzu.obecny kurz
        ukurzy+null:typKurzu.cenotvorba kurz-pro-cenotvorbu
        ukurzy+null:typKurzu.intrastat intrastat-kurz
        
        sumisteni+null:typUmisteniSklad.mistnost umisteni-ve-skladu-mistnost
        sumisteni+null:typUmisteniSklad.regal umisteni-ve-skladu-regal
        sumisteni+null:typUmisteniSklad.police umisteni-ve-skladu-police
    }
    
    array set evidence_easy_mapping {
        aadresar+null adresar
        abanspoj+null adresar-bankovni-ucet
        akontakty+null kontakt
        amisturc+null misto-urceni
        anaklakt+null naklad
        apsc+null psc
        askupfir+null skupina-firem
        asmerkod+null penezni-ustav
        astaty+null stat
        atypakt+null typ-aktivity
        atypnaklakt+null typ-nakladu
        catribut+null atribut
        ccenhlad+null cenova-uroven
        ccenik+null cenik
        cdodavatele+null dodavatel
        cenyview+null individualni-cenik
        ckusovnik+null kusovnik
        cmerjed+null merna-jednotka
        codberatele+null odberatel
        cpodobne+null podobne-zbozi
        cpoplatky+null poplatek
        cprislusenstvi+null prislustenstvi
        csady+null sady-a-komplety
        cskupcen+null cenikova-skupina
        cskupzboz+null skupina-zbozi
        ctypatribut+null typ-atributu
        ctypzbozi+null skupina-atributu
        danovaevidence+null danova-evidence
        dbsp+BAN bankovni-ucet
        dbsp+POK pokladna
        dbsp+SKL sklad
        dcislobal+null cislo-baliku
        ddoklfak+FAP faktura-prijata
        ddoklfak+PHL pohledavka
        ddoklfak+ZAV zavazek
        ddoklint+INT interni-doklad
        ddoklint+POK pokladni-pohyb
        ddoklobch+NAP nabidka-prijata
        ddoklobch+NAV nabidka-vydana
        ddoklobch+OBP objednavka-prijata
        ddoklobch+OBV objednavka-vydana
        ddoklobch+PPP poptavka-prijata
        ddoklobch+PPV poptavka-vydana
        ddoklsklad+SKL skladovy-pohyb
        delbanformat+null format-elektronickeho-bankovnictvi
        dformadopravy+null forma-dopravy
        dformauhradycis+null forma-uhrady
        dformauhrady+null prodejka-platba
        dformauhradyzauc+null forma-uhrady-zauctovani
        dkonsym+null konst-symbol
        dpolfak+FAP faktura-prijata-polozka
        dpolfak+FAV faktura-vydana-polozka
        dpolfak+PHL pohledavka-polozka
        dpolfak+ZAV zavazek-polozka
        dpolint+BAN banka-polozka
        dpolint+INT interni-doklad-polozka
        dpolint+POK pokladni-pohyb-polozka
        dpolobch+NAP nabidka-prijata-polozka
        dpolobch+NAV nabidka-vydana-polozka
        dpolobch+OBP objednavka-prijata-polozka
        dpolobch+OBV objednavka-vydana-polozka
        dpolobch+PPP poptavka-prijata-polozka
        dpolobch+PPV poptavka-vydana-polozka
        dpolsklad+SKL skladovy-pohyb-polozka
        dpolsml+null smlouva-polozka
        dprikazuhr+null prikaz-k-uhrade
        drady+ rada
        drady+BAN rada-banka
        drady+FAP rada-faktury-prijate
        drady+FAV rada-faktury-vydane
        drady+INT rada-interniho-dokladu
        drady+NAP rada-nabidky-prijate
        drady+NAV rada-nabidky-vydane
        drady+OBP rada-objednavky-prijate
        drady+OBV rada-objednavky-vydane
        drady+PHL rada-pohledavky
        drady+POK rada-pokladni-pohyb
        drady+PPP rada-poptavky-prijate
        drady+PPV rada-poptavky-vydane
        drady+SKL rada-skladovy-pohyb
        drady+ZAV rada-zavazku
        dradyrok+null rocni-rada
        dstavsml+null stav-smlouvy
        dtypdokl+FAP typ-faktury-prijate
        dtypdokl+INT typ-interniho-dokladu
        dtypdokl+NAP typ-nabidky-prijate
        dtypdokl+NAV typ-nabidky-vydane
        dtypdokl+OBP typ-objednavky-prijate
        dtypdokl+OBV typ-objednavky-vydane
        dtypdokl+PHL typ-pohledavky
        dtypdokl+POK typ-pokladni-pohyb
        dtypdokl+PPP typ-poptavky-prijate
        dtypdokl+PPV typ-poptavky-vydane
        dtypdokl+SKL typ-skladovy-pohyb
        dtypdokl+ZAV typ-zavazku
        dvazebtab+null vazba
        idodpodm+null intrastat-dodaci-podminky
        idruhdopr+null intrastat-druh-dopravy
        iintrmerjed+null intrastat-merna-jednotka
        ikrajurc+null intrastat-kraj-urceni
        inomen+null intrastat-kod-nomenklatury
        iobchtrans+null intrastat-obchodni-transakce
        izvlpoh+null intrastat-zvlastni-pohyb
        nnastav+null nastaveni
        nucetobdobi+null ucetni-obdobi
        podkladydph+null podklady-dph
        saldokdatu+ saldo-k-datu
        sinventura+null inventura
        skarty+null skladova-karta
        smapsklad+null mapovani-skladu
        spolinventura+null inventura-polozka
        srezervace+null rezervace
        stavskladu+null stav-skladu-k-datu
        svyrobnicislo+null vyrobni-cislo
        ucetnidenik+null ucetni-denik
        ucinnost+null cinnost
        uclendph+null cleneni-dph
        udphpren+null preneseni-dph
        umeny+null mena
        uobraty+null obrat
        usazbydph+null sazba-dph
        ustavzakazky+null stav-zakazky
        ustducty+null ucetni-osnova
        ustrediska+null stredisko
        utyporg+null typ-organizace
        utypucop+null predpis-zauctovani
        utypzakazky+null typ-zakazky
        uucty+null ucet
        uvyhzakazky+null hodnoceni-zakazky
        uzakazky+null zakazka
        wactclenstvi+null clenstvi-pro-workflow
        wactdefinice+null definice-workflow
        wactskupina+null skupina-pro-workflow
        wautcert+null certifikacni-autorita
        wdataprava+null pravo-viditelnosti
        wdotazy+null uzivatelsky-dotaz
        wdotazyparam+null uzivatelsky-dotaz-parametr
        wdotazyprop+null uzivatelsky-dotaz-vlastnost
        wfiltry+null filtr
        wparam+null parametr
        wpriloha+null priloha
        wreport+null report
        wrole+null role
        wskupvybklic+null skupina-stitku
        wstrom+null strom-koren
        wtexty+null text
        wuzel+null strom
        wuzivatel+null uzivatel
        wvazba+null uzivatelska-vazba
        wvazbatyp+null typ-uzivatelske-vazby
        wvazuzel+null strom-cenik
        wvybklice+null stitek
        wzastupnosti+null zastupnost
        wzurnal+null zurnal

        cstavcen+null stav-ceniku
        sumisteni+null umisteni-ve-skladu
        ukurzy+null kurz
    }
    
    set evidence null
    set evid_easy_key "$TG_table_name+[string trim $modul ']"
    
    if {[ info exists evidence_discriminators($evid_easy_key) ]} {
        set discriminator $evidence_discriminators($evid_easy_key)
        
        if {[ info exists NEW($discriminator) ]} {
            set disc_value "$NEW($discriminator)"
        } elseif {[ info exists OLD($discriminator) ]} {
            set disc_value "$OLD($discriminator)"
        }
        
        if {[ info exists disc_value ]} {
            set evid_disc_key "$evid_easy_key:$disc_value"
            
            if {[ info exists evidence_disc_mapping($evid_disc_key) ]} {
                set evidence '[ quote $evidence_disc_mapping($evid_disc_key) ]'
            }
        }
    }
    if {$evidence == "null"} {
        if {[info exists evidence_easy_mapping($evid_easy_key)]} {
            set evidence '[ quote $evidence_easy_mapping($evid_easy_key) ]'
        }
    }
    
    spi_exec "INSERT INTO changelog(tabulka, modul, evidence, id, operace, uzivatel, extids)
        VALUES ('[ quote $TG_table_name ]',
            $modul,
            $evidence,
            '[ quote $pkval ]',
            '[ quote $operation ]',
            '[ quote $user_name ]',
            $extid)"

    if {$TG_op == "DELETE"} {
        return [array get OLD]
    } else {
        return [array get NEW]
    }
$$ LANGUAGE pltcl VOLATILE SECURITY DEFINER;
ALTER FUNCTION logchanges() OWNER TO %admin%;


-- zvyseni verze databaze
INSERT INTO dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  VALUES (nextval('seq_dbversion'), 826, 't', %appVersion%, clock_timestamp());
