--------------------------------------------------------------------------------
-- Bug 37820 – [FlexiBee] Doplnit podporu pro měny do individuálního ceníku 
--------------------------------------------------------------------------------

-- agregacni funkce porovnavajici pole podle prvniho prvku a pri rovnosti podle druheho prvku 
CREATE OR REPLACE FUNCTION minceny_agg(numeric[], numeric[])
  RETURNS numeric[] AS
$BODY$
        SELECT (case when $1[1] < $2[1] then $1 else (case when $2[1] < $1[1] then $2 else 
                  (case when $2[2] < $1[2] then $2 else $1 end)
                end) end);
$BODY$
  LANGUAGE sql IMMUTABLE STRICT
  COST 100;
ALTER FUNCTION minceny_agg(numeric[], numeric[]) OWNER TO %firmaRole%;

DROP AGGREGATE IF EXISTS public.minceny (numeric[]);
CREATE AGGREGATE public.minceny (numeric[])
(
        sfunc    = public.minceny_agg,
        stype    = numeric[]
);
ALTER AGGREGATE minceny(numeric[]) OWNER TO %firmaRole%;

-- starou funkci odebereme
DROP FUNCTION IF EXISTS calcceny(numeric, numeric, numeric[], numeric[], numeric, integer, character varying, character varying, character varying, character varying, character varying, character varying, numeric, numeric[], numeric[]);

CREATE OR REPLACE FUNCTION calcceny(c_nakupCena numeric, c_cenazakl numeric,
  c_ceny numeric[], c_limmnoz numeric[], 
  o_cena numeric, o_firma boolean, o_skupcen boolean, o_idmena integer, kurz numeric,
  h_idcenhlad integer, h_typcenyvychozik varchar, h_typvypcenyk varchar, h_typcenyvychozi25k varchar, h_typvypceny25k varchar, 
  h_zaokrjakk varchar, h_zaokrnak varchar, h_proczakl numeric, h_procento numeric[], h_limmnoz numeric[])
  RETURNS numeric[] AS $$
declare
  vychozicena numeric;
  cena1 numeric;
  ceny numeric[];
  i integer;
  priorita numeric;
begin
  -- prvni prvek pole udava vyznam cen - ceny s nizsim cislem maji vzdy prednost i kdyz skutecna cena je vyssi
  --   1 - odberatel se zadanou konkretni firmou s konkretni menou
  --   2 - odberatel se zadanou konkretni firmou bez vyplnene meny
  --   3 - odberatel se zadanou konkretni skupinou s konkretni menou
  --   4 - odberatel se zadanou konkretni skupinou bez vyplnene meny
  --   5 - odberatel bez firmy s vyplnenou menou
  --   6 - odberatel bez firmy i bez meny
  --   7 - bez odberatele
  
  -- otestujeme zda nemame cenu pro konretniho odberatele - pokud je pro vsechny a nemame hladinu vracime ji taky bez vypoctu 
  if o_cena is not null and o_cena > 0 and (o_firma = true or o_skupcen = true or h_idcenhlad is null or h_typcenyvychozik = 'typCenyVychozi.nakCena') then
    cena1 = o_cena;
    priorita = 1;
    if (o_idmena is null) then
      priorita = 2;
      if (kurz is not null) then
        cena1 = round(cena1 * kurz, 6);
      end if;
    end if;
    
    if o_firma = false then
      priorita = priorita + 2;
      if o_skupcen = false then
        priorita = priorita + 2;
      end if;
    end if;
    
    return array[priorita, cena1, 0, 0, 0, 0, 0, 0, 0, 0, null];
  end if;

  -- neni zadana cenova hladina - bereme ceny primo z ceniku
  priorita = 7.0;
  if (h_idcenhlad is null) then
    for i in 1..4 loop
      ceny[i] = 0;
      if (c_limmnoz[i] > 0) then
        ceny[i] = c_ceny[i];
        if (o_idmena is null and kurz is not null) then      -- cena nebyla v mene a mame kurz - zapocteme ho
          ceny[i] = round(ceny[i] * kurz, 6);
        end if;
      end if;
    end loop;
    vychozicena = c_cenazakl;
    if (o_idmena is null and kurz is not null) then      -- cena nebyla v mene a mame kurz - zapocteme ho
      vychozicena = round(vychozicena * kurz, 6);
    end if;
    if vychozicena = 0 then
      vychozicena = null;
    end if;
    
    return array[priorita] || vychozicena || ceny || c_limmnoz || cast(h_idcenhlad as numeric);
  end if;
  
  vychozicena = (CASE h_typcenyvychozik
    WHEN 'typCenyVychozi.nakCena' THEN c_nakupCena
    WHEN 'typCenyVychozi.zaklCena' THEN c_cenazakl
    ELSE null
  END);
  
  if o_cena is not null and o_cena > 0 then   -- v pripade odberatele vezmeme cenu z nej - pripady kdy to nema smysl jsou odfiltrovany vyse
    vychozicena = o_cena;
    if o_idmena is null then
      priorita = 6.0;
    else
      priorita = 5.0;
    end if;
  end if; 
  
  cena1 = calccena(vychozicena, h_typvypcenyk, h_proczakl);

  -- zdroj pro pripadnou dalsi cenu urcujeme pred aplikaci kurzu, abz nebyl zmatek v tom jestli uz je kurz zapocten nebo ne 
  vychozicena = (CASE h_typcenyvychozi25k
    WHEN 'typCenyVychozi.nakCena' THEN c_nakupCena
    WHEN 'typCenyVychozi.zaklCena' THEN cena1
    ELSE null
  END);

  if (o_idmena is null and kurz is not null) then      -- cena nebyla v mene a mame kurz - zapocteme ho
    cena1 = round(cena1 * kurz, 6);
    vychozicena = round(vychozicena * kurz, 6);
  end if;
  
  -- kdyz bereme cenu z odberatele nema vypocet z nakupni ceny smysl - ignorujeme mnozstevni slevy
  if (o_cena is not null and o_cena > 0 and h_typcenyvychozi25k = 'typCenyVychozi.nakCena') then
    return array[priorita, cena1, 0, 0, 0, 0, 0, 0, 0, 0, cast(h_idcenhlad as numeric)];
  end if;
  
  cena1 = zaokrouhli(cena1, h_zaokrjakk, h_zaokrnak);
  for i in 1..4 loop
    ceny[i] = 0;
    if (h_limmnoz[i] > 0) then
      ceny[i] = zaokrouhli(calccena(vychozicena, h_typvypceny25k, h_procento[i]), h_zaokrjakk, h_zaokrnak);
    end if;
  end loop;
  
  return array[priorita, cena1] || ceny || h_limmnoz || cast(h_idcenhlad as numeric);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.calcceny(numeric, numeric, numeric[], numeric[], numeric, boolean, boolean, integer, numeric, integer, 
   varchar, varchar, varchar, varchar, varchar, varchar, numeric, numeric[], numeric[]) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION calcceny(_idcenik integer, _idcenhlad integer, _idfirma integer, _idskupcen integer)
  RETURNS numeric[] AS $$
declare
  cenik record;
  cenhlad record;
  odberatelcena numeric;
begin
  -- otestujeme zda nemame cenu pro konretniho odberatele
  odberatelcena = null;
  if _idfirma is not null then      -- zkusime najit jestli pro firmu neexistuje prima odberatelska cena
    select prodejcena into odberatelcena from codberatele where idcenik = _idcenik and idfirmy = _idfirma limit 1;
  end if;
  if _idskupcen is not null and odberatelcena is null then  -- zkusime najit jestli pro skupinu cen neexistuje prima odberatelska cena, pouze pokud uz nebyla nalezena pro firmu
    select prodejcena into odberatelcena from codberatele where idcenik = _idcenik and idskupcen = _idskupcen limit 1;
  end if;
  
  select * into cenik from ccenik where idcenik = _idcenik;

  if (_idcenhlad is null) then
    return calcceny(cenik.nakupCena, cenik.cenazakl,
    array[cenik.cena2, cenik.cena3, cenik.cena4, cenik.cena5], array[cenik.limmnoz2, cenik.limmnoz3, cenik.limmnoz4, cenik.limmnoz5],
    odberatelcena, _idfirma is not null, _idskupcen is not null, -1, null, null, null, null, null, null, null, null, null, null, null);
  end if;
  
  select * into cenhlad from ccenhlad where idcenhlad = _idcenhlad;
  
  return calcceny(cenik.nakupCena, cenik.cenazakl,
    array[cenik.cena2, cenik.cena3, cenik.cena4, cenik.cena5], array[cenik.limmnoz2, cenik.limmnoz3, cenik.limmnoz4, cenik.limmnoz5],
    odberatelcena, _idfirma is not null, _idskupcen is not null, -1, null,
    cenhlad.idcenhlad, cenhlad.typcenyvychozik, cenhlad.typvypcenyk, cenhlad.typcenyvychozi25k, cenhlad.typvypceny25k, 
    cenhlad.zaokrjakk, cenhlad.zaokrnak, cenhlad.proczakl, 
    array[cenhlad.procento2, cenhlad.procento3, cenhlad.procento4, cenhlad.procento5], array[cenhlad.limmnoz2, cenhlad.limmnoz3, cenhlad.limmnoz4, cenhlad.limmnoz5]);
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.calcceny(integer, integer, integer, integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 827, 't', %appVersion%, clock_timestamp());
