-----------------------------------------------------------------------
-- Bug 35012 - Rezervace na sady a komplety se chovají nelogicky 
------------------------------------------------------------------------

-- Funkce aktualizujici rezervaci pro obchodni polozku
CREATE OR REPLACE FUNCTION rezervujPolObch(_idCenik integer, _idSklad integer, _idPolObch integer, _mj numeric)
  RETURNS void AS $$
declare
  polObch record;
  _mnoz numeric(19,6);
  _idUcetObd integer;
  _rezer record;
  _idFirma integer;
  _idUziv integer;
  _stav varchar(50);
  _povolenyStav boolean;

  _issada integer;
  _sada record;
  _skladove boolean;
begin
  
  select skladove into _skladove from ccenik where idcenik = _idcenik;
  if(not _skladove) then
    -- pro neskladové zboží nemají rezervace smysl
    return;
  end if;
  
  -- pokud je ceník sadou chceme zarezervovat všechny součásti
  select count(*) into _issada from csady where idceniksada = _idcenik;
  if(_issada > 0) then
    for _sada in select * from csady where idceniksada = _idcenik loop
      perform rezervujPolObch(_sada.idCenik, _idsklad, _idpolobch, (_mj * _sada.mnozmj)); 
    end loop;
    return;
  end if;

  -- je-li  nastaveno, nechceme spouštět (nastavuje se ve fci srezervujzprijemky - ta sama si updatuje záznam v srezervace)
  if (getglotxvariable('rezerDisableTrigger') is not null) then 
      return; 
  end if; 
  
  select * into _rezer from srezervace where idPolObch = _idPolObch and idCenik = _idCenik and idSklad = _idSklad;
  
  _mnoz = _mj;
  if _rezer.idrezervace is not null then
    _mnoz = _mnoz + _rezer.mnozstvi;
  end if;
  
  if _mnoz > 0 then             -- pokud ma neco byt zarezervovane
    select * into polObch from dPolObch where idPolObch = _idPolObch;
    
    if _rezer.idrezervace is null then      -- pokud rezervace neexistuje vytvorime ji
      select stavUzivK into _stav from dDoklObch where idDoklObch = polObch.idDoklObch;   
      _povolenyStav := _stav is null or (_stav <> 'stavDoklObch.hotovo' and _stav <> 'stavDoklObch.vydano'  and _stav <> 'stavDoklObch.storno' );
      if(_povolenyStav) then
        select idFirmy, idUziv into _idFirma, _idUziv from dDoklObch where idDoklObch = polObch.idDoklObch;
      
        insert into srezervace (lastupdate, idpolobch, iduziv, idfirmy, idcenik, idsklad, mnozstvi, datumod)
              values (now(), _idPolObch, _idUziv, _idFirma, _idCenik, _idSklad, _mnoz, date_trunc('minute', now()));
      end if;
    else
      update srezervace set mnozstvi = _mnoz where idrezervace = _rezer.idRezervace;
    end if;
  else
    if _rezer.idrezervace is not null then    -- pokud rezervace byt nema a je smazeme ji
      delete from srezervace where idrezervace = _rezer.idRezervace;
    end if;
  end if;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.rezervujPolObch(integer, integer, integer, numeric) OWNER TO %firmaRole%;


-- trigger volany pri vytvoreni nebo zmene obchodni polozky
CREATE OR REPLACE FUNCTION dpolobchupdaterezer()
  RETURNS trigger AS $$
declare
  _sada record;
  _issada integer;
  _rezervovatMj numeric(19,6);
begin
  if NEW.storno = true then
    -- smazeme rezervaci pro polozku
    delete from srezervace where idpolobch = NEW.idpolobch;
    return NEW;
  end if;

  if (tg_op = 'UPDATE') then
    if OLD.idBspSkl is not null and OLD.rezervovat and OLD.storno = false then
      -- nejdříve zrušíme rezervace které jsou již vytvořeny
      perform rezervujPolObch(OLD.idCenik, OLD.idBspSkl, OLD.idPolObch, -OLD.rezervovatMj);
    end if;
  end if;
  
  if NEW.idBspSkl is not null and NEW.rezervovat and NEW.storno = false then 
    _rezervovatMj = NEW.rezervovatMj;
    if(NEW.rezervovatMj > NEW.mnozMj - NEW.mnozMjReal) then
      _rezervovatMj = NEW.mnozMj - NEW.mnozMjReal;
    end if;
    perform rezervujPolObch(NEW.idCenik, NEW.idBspSkl, NEW.idPolObch, _rezervovatMj); 
  end if; 
 
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolobchupdaterezer() OWNER TO %firmaRole%;


-- Funkce odrezervujici dane mnozstvi z pro danou cenikou polozku na sklade
CREATE OR REPLACE FUNCTION odrezervuj(_idFirmy integer, _idCenik integer, _idSkladu integer, vyskladni numeric(19,6))
  RETURNS void AS $$
declare
  _idRezer integer;
  _mnozstvi numeric(19,6);
  _skladMj numeric(19,6);
  _vyskl numeric(19,6);

  _issada integer;
  _sada record;
begin
  -- pokud je ceník sadou chceme zarezervovat všechny součásti
  select count(*) into _issada from csady where idceniksada = _idcenik;
  if(_issada > 0) then
    for _sada in select * from csady where idceniksada = _idcenik loop
      perform odrezervuj(_idfirmy, _sada.idcenik, _idskladu, (vyskladni * _sada.mnozmj));
    end loop;
    return;
  end if;

  _vyskl = vyskladni;
  
  perform putglotxvariable('rezerDisableTrigger', '1');
  for _idRezer, _mnozstvi, _skladMj in select idRezervace, mnozstvi, skladmj from sRezervace where idfirmy = _idFirmy and idCenik = _idCenik and idSklad = _idSkladu order by idrezervace loop
    if _vyskl < _skladMj or _skladMj <> _mnozstvi then
      update sRezervace set mnozstvi = (mnozstvi-_vyskl), skladmj = (skladmj-_vyskl) where idRezervace = _idRezer;
      
      perform removeglotxvariable('rezerDisableTrigger');
      return;
    else
      delete from sRezervace where idRezervace = _idRezer;
      
      _vyskl = _vyskl - _mnozstvi;
    end if;
  end loop;
  
  if (_vyskl > 0) then        -- nemelo by nastat, ale pokud zbyde, odrezervujeme i to co nebylo rezervoveny primo na sklade
    for _idRezer, _mnozstvi in select idRezervace, mnozstvi from sRezervace where idfirmy = _idFirmy and idCenik = _idCenik and idSklad = _idSkladu order by idrezervace loop
      if _vyskl < _mnozstvi then
        update sRezervace set mnozstvi = (mnozstvi-_vyskl) where idRezervace = _idRezer;
      
        perform removeglotxvariable('rezerDisableTrigger');
        return;
      else
        delete from sRezervace where idRezervace = _idRezer;
      
        _vyskl = _vyskl - _mnozstvi;
      end if;
    end loop;
  end if;
  
  perform removeglotxvariable('rezerDisableTrigger');
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.odrezervuj(integer, integer, integer, numeric) OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION dgetrezervovano(_cenik integer, _sklad integer, i_firmaid integer)
  RETURNS numeric AS $$  
declare  
  _celkemRezerMj numeric(19,6);  
  _pomoc numeric(19,6);

  _issada integer;
  _sada record;
  _skladove boolean;
begin  

  select skladove into _skladove from ccenik where idcenik = _cenik;
  if(not _skladove) then
    -- pro neskladové zboží nemají rezervace smysl
    return 0.0;
  end if;
  

  -- pokud je ceník sadou chceme zarezervovat všechny součásti
  select count(*) into _issada from csady where idceniksada = _cenik;
  if(_issada > 0) then
    _celkemRezerMj = 0.0;
    for _sada in select * from csady where idceniksada = _cenik loop
      select dgetrezervovano(_sada.idcenik, _sklad, i_firmaid) into _pomoc;
      if(_sada.mnozMj <> 0) then
        if((_pomoc / _sada.mnozMj) > _celkemRezerMj) then
          _celkemRezerMj = _pomoc / _sada.mnozMj;
        end if;
      end if;
    end loop;
    RETURN _celkemRezerMj;   
  end if;

  -- vymazeme jiz neplatne rezervace  
  delete from srezervace where datumdo is not null and datumdo < date(now());  
  -- zjistime kolik je celkem rezervovano - od/do ignorujeme (bug 25844, 25789), muselo by se zohlednovat uz pri plneni pole skladmj a nebo toto pole idealne vyhodit 
  select coalesce(sum(skladmj), 0) into _celkemRezerMj from srezervace where idcenik = _cenik and idsklad = _sklad and idfirmy <> i_firmaid; 
    
  RETURN _celkemRezerMj;  
end  
$$ LANGUAGE plpgsql;
ALTER FUNCTION dgetrezervovano(integer, integer, integer) OWNER TO %firmaRole%;


/* i_iddoklobch - pole, pokud hledáme pro více obchodních dokladů. Pokud hledáme jen pro jeden obch. doklad, bude délky 1.
 *     Pokud nehledáme pro žádný obchodní doklad, bude délky 1 s jediným elementem "-1".
 * */
CREATE OR REPLACE FUNCTION dgetstavnaskladeaktualni(i_idcenik integer, i_idsklad integer, i_datum date,
 i_idfirma integer, i_iddoklobch integer[], i_novyDoklad boolean)
  RETURNS numeric AS $$
declare
  _stavMj numeric(19,6); 
  _firmaRezerMj numeric(19,6); 
  _celkemRezerMj numeric(19,6); 

  _issada integer;
  _sada record;
  _pomoc numeric(19,6);
  _skladove boolean;
begin
  -- pokud je ceník sadou chceme řešit všechny součásti
  select count(*) into _issada from csady where idceniksada = i_idcenik;
  if(_issada > 0) then
    _stavMj = 9999999999999.0; -- vic uz pretejka 
    select skladove into _skladove from ccenik where idcenik = i_idcenik; 
    if(_skladove) then 
      for _sada in select * from csady where idceniksada = i_idcenik loop
        select dgetstavnaskladeaktualni(_sada.idcenik, i_idsklad, i_datum, i_idfirma, i_iddoklobch, i_novydoklad) into _pomoc;
        if(_sada.mnozMj != 0.0) then
          _pomoc = _pomoc / _sada.mnozMj;
        end if;
        _stavMj = minValue(_stavMj, _pomoc); 
      end loop;
      return _stavMj;
    end if;
  end if;

  SELECT dgetstavnaskladeaktualni(i_idcenik, i_idsklad, i_datum) INTO _stavMj; 
  
  -- zjistime kolik je celkem rezervovano - od/do ignorujeme (bug 25844, 25789), muselo by se zohlednovat uz pri plneni pole skladmj a nebo toto pole idealne vyhodit
  select coalesce(sum(skladmj), 0) into _celkemRezerMj from srezervace where idcenik = i_idcenik and idsklad = i_idsklad;
   
  IF (i_idfirma > 0) THEN 
    -- array_upper - zjištění délky pole
    -- podmínka - viz možný obsah i_iddoklobch v hlavičce metody, Posgtgresu se pole indexují od 1 -> [1]
    IF ((array_upper(i_iddoklobch,1) = 1 and i_iddoklobch[1] > 0) or array_upper(i_iddoklobch,1) > 1) THEN 
        -- zjistime kolik ma firma rezervovano  
        -- fce any - pro hledání "in array"
        select coalesce(sum(r.skladmj), 0) into _firmaRezerMj from srezervace r 
            left join dpolobch po on r.idpolobch = po.idpolobch
            where r.idfirmy = i_idfirma and r.idcenik = i_idcenik and r.idsklad = i_idsklad and (po.iddoklobch is null or po.iddoklobch  = any(i_iddoklobch)); 
    ELSIF (i_novyDoklad) THEN
            -- jedná se o novou objednávku - zjistime kolik ma firma rezervovano, ale v tomto pripade jen rezervace bez dokladu
        select coalesce(sum(r.skladmj), 0) into _firmaRezerMj from srezervace r  
            left join dpolobch po on r.idpolobch = po.idpolobch 
            where r.idfirmy = i_idfirma and r.idcenik = i_idcenik and r.idsklad = i_idsklad and po.iddoklobch is null;  
    ELSE
        select coalesce(sum(skladmj), 0) into _firmaRezerMj from srezervace where idfirmy = i_idfirma and idcenik = i_idcenik and idsklad = i_idsklad; 
    END IF;
  ELSE 
    _firmaRezerMj := 0; 
  END IF; 
   
  -- spocteme kolik mu muzeme dat 
  _stavMj := minValue(_stavMj, maxValue(_stavMj - _celkemRezerMj, 0) + _firmaRezerMj); 
   
  RETURN _stavMj; 
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dgetstavnaskladeaktualni(integer, integer, date, integer, integer[],boolean) OWNER TO %firmaRole%;


--------------------------------------------------------------------------------
-- Bug 44711 - Při přepnutí objednávky do stavu storno se nezruší rezervace   
--------------------------------------------------------------------------------

-- trigger volany pri zmene obchodniho dokladu, kontroluje zmenu firmy
CREATE OR REPLACE FUNCTION ddoklobchupdaterezer()
  RETURNS trigger AS $$
begin
  if (OLD.idFirmy <> NEW.idFirmy) then
    update srezervace set idFirmy = NEW.idFirmy where idPolObch in (select idPolObch from dPolObch where idDoklObch = NEW.idDoklObch);
  end if;
  
  if (not comparevarchar(OLD.stavUzivK, NEW.stavUzivK) and NEW.stavUzivK = 'stavDoklObch.storno') then
    delete from srezervace where idpolobch in (select idpolobch from dpolobch where iddoklobch = NEW.idDoklObch);
  end if;
   
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ddoklobchupdaterezer() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
values (nextval('seq_dbversion'), 994, 't', %appVersion%, clock_timestamp());
