<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:java="http://xml.apache.org/xalan/java" exclude-result-prefixes="java">
  <xsl:output method="xml" indent="yes" encoding="utf-8" />
  <xsl:template match="MoneyData">
    <winstrom version="1.0" source="s3">
      <!-- kontrola a pripadne zalozeni dobropisu -->
      <xsl:if test="./SeznamFaktPrij/FaktPrij[Dobropis>0]">
        <typ-faktury-prijate>
          <id>ext:mS3tfap:MS3DOBROPIS</id>
          <id>code:MS3DOBROPIS</id>
          <kod>MS3DOBROPIS</kod>
          <nazev>Dobropis - nedaňový doklad</nazev>
          <modul>FAP</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.dobropis</typDoklK>
        </typ-faktury-prijate>
      </xsl:if>

      <!-- kontrola a pripadne zalozeni proforma -->
      <xsl:if test="./SeznamFaktPrij/FaktPrij[Druh='F']">
        <typ-faktury-prijate update="ignore">
          <id>ext:mS3tfap:MS3PROFORMA</id>
          <id>code:MS3PROFORMA</id>
          <kod>MS3PROFORMA</kod>
          <nazev>Proforma - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAP</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.proforma</typDoklK>
        </typ-faktury-prijate>
      </xsl:if>

      <!-- kontrola a pripadne zalozeni proforma -->
      <xsl:if test="./SeznamFaktPrij/FaktPrij[Druh='L']">
        <typ-faktury-prijate update="ignore">
          <id>ext:mS3tfap:MS3ZALOHA</id>
          <id>code:MS3ZALOHA</id>
          <kod>MS3ZALOHA</kod>
          <nazev>Záloha - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAP</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.zalohFaktura</typDoklK>
        </typ-faktury-prijate>
      </xsl:if>
      
      <!-- zalozeni typ dokladu zbylych fap -->
      <typ-faktury-prijate update="ignore">
          <id>ext:mS3tfap:MS3FAKTURA</id>
          <id>code:MS3FAKTURA</id>
          <kod>MS3FAKTURA</kod>
          <nazev>Faktura - nedaňový doklad pro doklady z Money S3</nazev>
          <modul>FAP</modul>
          <ucetni>false</ucetni>
          <typDoklK>typDokladu.faktura</typDoklK>
      </typ-faktury-prijate>
      
      <xsl:apply-templates select="SeznamFaktPrij"/>
    </winstrom>
  </xsl:template>
  
<!-- template pro spolecne veci jak vydanych tak prijatych faktur -->
  <xsl:template name="dokladCommon">
    <kod><xsl:value-of select="Doklad"/></kod>
    <popis><xsl:value-of select="Popis"/></popis>
    <datVyst><xsl:value-of select="java:flexibee.Tool.parseDate(Vystaveno, 'y-M-d')"/></datVyst>
    <datSplat><xsl:value-of select="java:flexibee.Tool.parseDate(Splatno, 'y-M-d')"/></datSplat>
    <xsl:if test="DatUcPr">
      <duzpPuv><xsl:value-of select="java:flexibee.Tool.parseDate(DatUcPr, 'y-M-d')"/></duzpPuv>
    </xsl:if>
    <xsl:if test="PlnenoDPH">
      <duzpUcto><xsl:value-of select="java:flexibee.Tool.parseDate(PlnenoDPH, 'y-M-d')"/></duzpUcto>
    </xsl:if>
    <datUhr><xsl:value-of select="java:flexibee.Tool.parseDate(Uhrazeno, 'y-M-d')"/></datUhr>
    
    <xsl:if test="KonstSym">
      <konSym if-not-found="nearest-invalid">code:<xsl:value-of select="java:flexibee.Tool.padNumber(4,KonstSym)"/></konSym>
    </xsl:if>

    <varSym><xsl:value-of select="VarSymbol"/></varSym>

    <formaUhrK>
    <xsl:choose><xsl:when test="Uhrada = 'převodem'">formaUhr.prevod</xsl:when>
    <xsl:otherwise>formaUhr.nespec</xsl:otherwise>
    </xsl:choose>
    </formaUhrK>
    
    <szbDphSniz><xsl:value-of select="java:flexibee.Tool.parseMoney(SazbaDPH1)"/></szbDphSniz>
    <szbDphZakl><xsl:value-of select="java:flexibee.Tool.parseMoney(SazbaDPH2)"/></szbDphZakl>
    
    <xsl:comment>castky</xsl:comment>
    <!-- castky -->
    <sumOsv><xsl:value-of select="./SouhrnDPH/Zaklad0"/></sumOsv>
    <sumZklSniz><xsl:value-of select="./SouhrnDPH/Zaklad5"/></sumZklSniz>
    <sumZklZakl><xsl:value-of select="./SouhrnDPH/Zaklad22"/></sumZklZakl>
    <sumDphSniz><xsl:value-of select="./SouhrnDPH/DPH5"/></sumDphSniz>
    <sumDphZakl><xsl:value-of select="./SouhrnDPH/DPH22"/></sumDphZakl>
  </xsl:template>
  
  <xsl:template name="dodOdb">
    <xsl:if test="./DodOdb/GUID">
      <firma>ext:mS3adr:<xsl:value-of select="substring(./DodOdb/GUID, 2, string-length(./DodOdb/GUID)-2)"/></firma>
    </xsl:if>
    <nazFirmy><xsl:value-of select="./DodOdb/FaktNazev"/></nazFirmy>
    <ulice><xsl:value-of select="./DodOdb/FaktAdresa/Ulice"/></ulice>
    <mesto><xsl:value-of select="./DodOdb/FaktAdresa/Misto"/></mesto>
    <psc><xsl:value-of select="./DodOdb/FaktAdresa/PSC"/></psc>
    <ic><xsl:value-of select="./DodOdb/ICO"/></ic>
    <dic><xsl:value-of select="./DodOdb/DIC"/></dic>
    
    <buc><xsl:value-of select="./DodOdb/Ucet"/></buc>
    <xsl:if test="./DodOdb/KodBanky != '' and java:flexibee.MigrationTool.existKodable('smerKod', ./DodOdb/KodBanky)">
        <smerKod>code:<xsl:value-of select="./DodOdb/KodBanky"/></smerKod>
    </xsl:if>
    <banSpojDod if-not-found="null">ext:mS3bu:<xsl:value-of select="concat(substring(./DodOdb/GUID, 2, string-length(./DodOdb/GUID)-2), ./DodOdb/KodBanky)"/></banSpojDod>
  </xsl:template>
  
<!-- prijate faktury -->
  <xsl:comment>prijate faktury</xsl:comment>
  <xsl:template match="SeznamFaktPrij">
    <xsl:apply-templates select="FaktPrij"/>
  </xsl:template>
  
  <xsl:template match="FaktPrij">
    <xsl:if test="./Doklad">
    <xsl:if test="./DodOdb/Ucet != '' and ./DodOdb/KodBanky != '' and ./DodOdb/Banka != ''">
      <adresar-bankovni-ucet update="ignore">
        <id>ext:mS3bu:<xsl:value-of select="concat(substring(./DodOdb/GUID, 2, string-length(./DodOdb/GUID)-2),./DodOdb/KodBanky)"/></id>
        <buc><xsl:value-of select="./DodOdb/Ucet"/></buc>
        <smerKod><xsl:value-of select="java:flexibee.MigrationTool.getValidOrNearestIdent('smerKod', ./DodOdb/KodBanky)"/></smerKod>
        <nazBanky><xsl:value-of select="./DodOdb/Banka"/></nazBanky>
        <firma>ext:mS3adr:<xsl:value-of select="substring(./DodOdb/GUID, 2, string-length(./DodOdb/GUID)-2)"/></firma>
      </adresar-bankovni-ucet>
    </xsl:if>
    
    <faktura-prijata>
      <id>ext:mS3fap:<xsl:value-of select="Doklad"/></id>
      <xsl:call-template name="dokladCommon"/>
      
      <cisDosle><xsl:value-of select="Doklad"/></cisDosle>
      
      <typDokl>
        <xsl:choose>
          <xsl:when test="Dobropis = '1'">code:MS3DOBROPIS</xsl:when>
          <xsl:when test="Druh = 'F'">code:MS3PROFORMA</xsl:when>
          <xsl:when test="Druh = 'L'">code:MS3ZALOHA</xsl:when>
          <xsl:when test="Druh = 'N'">code:MS3FAKTURA</xsl:when>
          <xsl:otherwise>code:MS3FAKTURA</xsl:otherwise>
        </xsl:choose>
      </typDokl>
      <xsl:comment>dodavatel</xsl:comment>
      <!-- dodavatel -->
      <xsl:if test="./DodOdb">
        <xsl:call-template name="dodOdb"/>
      </xsl:if>
      
      <bezPolozek>true</bezPolozek>
    </faktura-prijata>
    </xsl:if>
  </xsl:template>
</xsl:stylesheet>
